<?php 
	if (!defined("ENV")) exit();
	
	set_time_limit(1000);
	
	$action = isset($_GET["action"]) ? $_GET["action"] : "";
	$generateType = isset($_GET["generateType"]) ? $_GET["generateType"] : "all";
	$generateTarget = isset($_GET["generateTarget"]) ? $_GET["generateTarget"] : "new";
	
	$pathImages = "images/products/";
	$pathImagesThumbs = "images/products/thumbs/";
	$pathImagesProductPage = "images/products/preview/";
	$pathImagesSecondary = "images/products/secondary/";
	
	switch($action)
	{
		/**
		 * Scan folder for images, and cache secondary images
		 */
		case "scanImages" :
		{
			/**
			 * Scan for primary images
			 */
			$imagesPrimary = array();
			if (in_array($generateType, array("all", "thumbs", "products")))
			{
				$d = dir($pathImages);
				while (false !== ($entry = $d->read()))
				{
   					$filePath = $pathImages.$entry;
   					if (is_file($filePath))
   					{
   						$fileInfo = pathinfo(strtolower($filePath));
   						if (in_array($fileInfo["extension"], array("jpg", "jpeg", "png", "gif")))
   						{
   							$imagesPrimary[] = $filePath;
   						}
   					}
				}
				$d->close();
			}
			
			/**
			 * Scan for secondary images
			 */
			$imagesSecondary = array();
			if (in_array($generateType, array("all", "secondary")))
			{
				$db->query("SELECT iid, type FROM ".DB_PREFIX."products_images");
				while (($img = $db->movenext()) != false)
				{
					$imagesSecondary[] = $pathImagesSecondary.$img["iid"].".".$img["type"]; 
				}
			}

			/**
			 * SAVE SCAN RESULT
			 */
			$primaryCount = count($imagesPrimary) ;
			$secondaryCount = count($imagesSecondary);
			
			$response = array(
				"primaryCount"=>$primaryCount,
				"secondaryCount"=>$secondaryCount
			);
			
			if ($primaryCount > 0 || $secondaryCount > 0)
			{
				$_SESSION["thumb_generator"] = array(
					"primaryCount" => $primaryCount,
					"secondaryCount" => $secondaryCount,
					"imagesPrimary" => $imagesPrimary,
					"imagesSecondary" => $imagesSecondary,
					"generateType" => $generateType,
					"generateTarget" => $generateTarget,
					"totalCount" => $primaryCount + $secondaryCount,
					"primaryPosition" => 0,
					"secondaryPosition" => 0
				);
			}
			
			echo json_encode($response);
			break;
		}
		
		/**
		 * GENERATE THUMBS
		 */
		case "generateThumbs" :
		{
			$atOnce = rand(10, 25);
			
			// Get data from session
			$primaryPosition = $_SESSION["thumb_generator"]["primaryPosition"];
			$secondaryPosition = $_SESSION["thumb_generator"]["secondaryPosition"];
			$primaryCount =$_SESSION["thumb_generator"]["primaryCount"];
			$secondaryCount = $_SESSION["thumb_generator"]["secondaryCount"];
			
			$generateType = $_SESSION["thumb_generator"]["generateType"];
			$generateTarget = $_SESSION["thumb_generator"]["generateTarget"];
			
			//Check what is already done
			$primaryDone = $primaryPosition + 1 >= $primaryCount;
			$secondaryDone = $secondaryPosition + 1 >= $secondaryCount;
			
			//Generate primary images
			if (!$primaryDone)
			{
				//Preload all products ids from database
				$products = array();
				$db->query("SELECT pid, product_id FROM ".DB_PREFIX."products");
				while (($product = $db->movenext()) != false)
				{
					$products[strtolower(trim($product["product_id"]))] = $product["pid"];
				}
				
				$fileToProcess = array();
				
				$where = "";
				
				//Loop images to be processed
				for ($i = $primaryPosition; $i < $primaryPosition + $atOnce ; $i++)
				{
					//check is it time to stop
					if ($i >= $primaryCount)
					{
						$i--;
						$primaryDone = true;
						break;
					}
					
					$filePath = $_SESSION["thumb_generator"]["imagesPrimary"][$i];
					
					$pathInfo = pathinfo($filePath);
					if (!isset($pathInfo["name"])) $pathInfo["name"] = substr($pathInfo["basename"], 0, strlen($pathInfo["basename"]) - strlen($pathInfo["extension"]) - 1);
				
					if (isset($products[$pathInfo["name"]]))
					{
						$error = "";
						$pid = $products[$pathInfo["name"]];
						
						//generate thumb image
						if (($generateType == "all" || $generateType == "thumbs"))
						{
							if (($generateTarget == "all" || ($generateTarget == "new" && !is_file($pathImagesThumbs.$pathInfo["name"].".jpg"))))
							{
								if (!DEMO_LOCK)
								{
									if (getImageThumb($filePath, $pathImagesThumbs.$pathInfo["name"].".jpg", $settings["CatalogThumbSize"], $settings["CatalogThumbType"], $error))
									{
										$where .= ($where == "" ? "" : " OR ")."pid='".$pid."'";								
									}
								}
							}
						}

						//generate product page image
						if (($generateType == "all" || $generateType == "thumbs"))
						{
							if (($generateTarget == "all" || ($generateTarget == "new" && !is_file($pathImagesProductPage.$pathInfo["name"].".jpg"))))
							{
								if (!DEMO_LOCK)
								{
									if (@getImageThumb($filePath, $pathImagesProductPage.$pathInfo["name"].".jpg", $settings["CatalogThumb2Size"], $settings["CatalogThumb2Type"], $error))
									{
										$where .= ($where == "" ? "" : " OR ")."pid='".$pid."'";
									}
								}		
							}
						}
					}
				}
				
				//update producs locations
				if (!DEMO_LOCK && $where != "")
				{
					$db->query("UPDATE ".DB_PREFIX."products SET image_location='Local', image_url='' WHERE ".$where);
				}
				$primaryPosition = $i;
				
				unset($products);
			}
			
			//Generate secondary images
			if ($primaryDone && !$secondaryDone)
			{
				for ($i = $secondaryPosition; $i < $secondaryPosition + $atOnce ; $i++)
				{
					if ($i >= $secondaryCount)
					{
						$i--;
						$secondaryDone = true;
						break;
					}
					$filePath = $_SESSION["thumb_generator"]["imagesSecondary"][$i];
					$pathInfo = pathinfo($filePath);
					if (!isset($pathInfo["name"])) $pathInfo["name"] = substr($pathInfo["basename"], 0, strlen($pathInfo["basename"]) - strlen($pathInfo["extension"]) - 1);
					
					$iid = $pathInfo["name"];
						
					//generate thumb image
					if (($generateType == "all" || $generateType == "secondary"))
					{
						$pathGenerated = $pathImagesSecondary.$iid."_thumb.jpg";
						if (($generateTarget == "all" || ($generateTarget == "new" && !is_file($pathGenerated))))
						{
							$error = "";
							if (!DEMO_LOCK)
							{
								getImageThumb($filePath, $pathGenerated, $settings["CatalogThumbSize"], $settings["CatalogThumbType"], $error);
							}
						}
					}					
				}
				$secondaryPosition = $i;
			}
			
			if ($primaryDone && $secondaryDone)
			{
				unset($_SESSION["thumb_generator"]);
			}
			else
			{
				$_SESSION["thumb_generator"]["primaryPosition"] = $primaryPosition;
				$_SESSION["thumb_generator"]["secondaryPosition"] = $secondaryPosition;
			}
			
			$response = array(
				"done" => $primaryDone && $secondaryDone,
				"progress" =>  round(100 / ($primaryCount + $secondaryCount) * ($primaryPosition + $secondaryPosition), 2),
				"progressCount" => $primaryPosition + $secondaryPosition,
				"totalCount" => $primaryCount + $secondaryCount
			);
			
			echo json_encode($response);
			
			break;
		}
	}
