<?php
if(!class_exists("DobaAPI")) require_once("content/classes/class.doba.php");
/**
 * Doba Order API Class
 *
 */
class DobaOrderAPI extends DobaAPI 
{
	
	/**
	 * Order lookup
	 *
	 * @param array $shipping_address
	 * @param array $items
	 * @return mixed
	 */
	public function orderLookup($shipping_address, $items)
	{
		$xml = 
			"<action>orderLookup</action>".
			"<shipping_firstname>-</shipping_firstname>".
			"<shipping_lastname>".utf8_encode($shipping_address["name"])."</shipping_lastname>".
			"<shipping_street>".utf8_encode($shipping_address["address1"].($shipping_address["address2"]!=""?(", ".$shipping_address["address1"]):""))."</shipping_street>".
			"<shipping_city>".utf8_encode($shipping_address["city"])."</shipping_city>".
			"<shipping_state>".utf8_encode($shipping_address["state_abbr"])."</shipping_state>".
			"<shipping_postal>".utf8_encode($shipping_address["zip"])."</shipping_postal>".
			"<shipping_country>".utf8_encode($shipping_address["country_iso_a2"])."</shipping_country>".
			"<ip_address>".$_SERVER['REMOTE_ADDR']."</ip_address>".
			"<items>";
		
		foreach ($items as $item)
		{
			$xml.=
				"<item>".
					"<item_id>".utf8_encode($item["item_id"])."</item_id>".
					"<quantity>".utf8_decode($item["quantity"])."</quantity>".
				"</item>";
		}
		
		$xml.=
			"</items>";
					
		$r = $this->call($xml);
		
		return $r ? $r["dce"][0]["response"][0] : false;
	}
	
	/**
	 * Get shipping price for current order
	 *
	 * @param array $shipping_address
	 * @param array $items
	 */
	public function getShippingQuote($shipping_address, $items)
	{
		$result = $this->orderLookup($shipping_address, $items);
		if($result && isset($result["shipping_fees"][0]))
		{
			$a = $result["shipping_fees"][0];
			return $a;
		}
		$this->last_error = $this->last_error != "" ? $this->last_error : "Error processing shipping data.";
		return false;
	}
	
	/**
	 * Creates order
	 *
	 * @param array $shipping_address
	 * @param array $items
	 * @return mixed
	 */
	public function createOrder($shipping_address, $items)
	{
		$xml = 
			"<action>createOrder</action>".
			"<shipping_firstname>-</shipping_firstname>".
			"<shipping_lastname>".utf8_encode($shipping_address["name"])."</shipping_lastname>".
			"<shipping_street>".utf8_encode($shipping_address["address1"].($shipping_address["address2"]!=""?(", ".$shipping_address["address1"]):""))."</shipping_street>".
			"<shipping_city>".utf8_encode($shipping_address["city"])."</shipping_city>".
			"<shipping_state>".utf8_encode($shipping_address["state_abbr"])."</shipping_state>".
			"<shipping_postal>".utf8_encode($shipping_address["zip"])."</shipping_postal>".
			"<shipping_country>".utf8_encode($shipping_address["country_iso_a2"])."</shipping_country>".
			"<ip_address>".$_SERVER['REMOTE_ADDR']."</ip_address>".
			"<items>";
		
		foreach ($items as $item)
		{
			$xml.=
				"<item>".
					"<item_id>".utf8_encode($item["item_id"])."</item_id>".
					"<quantity>".utf8_decode($item["quantity"])."</quantity>".
				"</item>";
		}
		
		$xml.=
			"</items>";
					
		$r = $this->call($xml);
		
		if (isset($r["dce"][0]["response"][0]["outcome"][0]) && isset($r["dce"][0]["response"][0]["order_id"][0]))
		{
			if ($r["dce"][0]["response"][0]["outcome"][0] == "success")
			{
				return $r["dce"][0]["response"][0];
			}
		}
		
		return false;
	}
}