<?php 

define("ORDER_ADD_ITEM", "AddItem");
define("ORDER_REMOVE_ITEM", "RemoveItem");
define("ORDER_CLEAR_ITEMS", "ClearItems");
define("ORDER_UPDATE_ITEMS", "UpdateItems");
define("ORDER_RE_UPDATE_ITEMS", "ReUpdateItems");
define("ORDER_CONTINUE_SHOPPING", "ContinueShopping");
define("ORDER_CHECK_PROMO", "CheckPromoCode");
define("ORDER_ONE_PAGE_CHECKOUT", "OnePageCheckout");
define("ORDER_BILLING_AND_SHIPPING_ADDRESS", "OrderBillingAndShippingAddress");
define("ORDER_UPDATE_BILLING_AND_SHIPPING_ADDRESS", "OrderUpdateBillingAndShippingAddress");
define("ORDER_UPDATE_SHIPPING_METHOD", "OrderUpdateShippingMethod");
define("ORDER_STORE_PAYPAL_EC_INFO", "StorePayPalECInfo");
define("ORDER_PROCESS_PAYMENT", "ProcessPayment");
define("ORDER_PROCESS_PAYMENT_VALIDATION", "ProcessPaymentValidation");
define("ORDER_COMPLETE_ORDER", "CompleteOrder");
define("ORDER_WISHLIST_TO_CART", "OrderWishlistToCart");

class ORDER
{
	// order properties
	private $_db = null;
	private $_settings = null;
	private $_user = null;
	
	public $oid = 0, $order_num = 0;
	public $errors;
		
	//products in cart
	//var $productsCount = 0;
	public $itemsCount = 0;
	
	//subtotal amount
	public $subtotalAmount = 0;
	public $subtotalAmountWithTax = 0;
	
	//discount
	public $discountAmount = 0;
	public $discountValue = 0;
	public $discountType = "";
	
	//promo discount
	public $promoDiscountAmount = 0;
	public $promoDiscountValue = 0;
	public $promoDiscountType = "";
	
	//tax
	public $taxCountryId = 0;
	public $taxStateId = 0;
	public $taxAmount = 0;
	public $taxRates = array("0"=>array("multiplier"=>1, "tax_rate"=>0, "description"=>""));
	
	//handling - common
	public $handlingSeparated = 0;
	public $handlingFee = 0.00;
	public $handlingFeeType = "";	
	public $handlingText = "";
	public $handlingAmount = 0.00;
	//handling - tax-related
	public $handlingTaxable = 0; // $settings["ShippingHandlingTaxable"]
	public $handlingTaxClassId = 0; // $settings["ShippingHandlingTaxClassId"]
	public $handlingTaxRate = 0.00;
	public $handlingTaxDescription = "";
	public $handlingTaxAmount = 0.00;
	
	//shipping
	public $shippingCompleted = 0;
	public $shippingIsFree = 0;
	public $shippingDigital = 0;
	public $shippingSSID = "0";
	public $shippingCarrierID = "";
	public $shippingMethodID = "";
	public $shippingCMName = "";
	public $shippingAddress = array();
	public $shippingAmount = 0.00;	
	
	public $shippingTaxable = 0;
	public $shippingTaxClassId = 0;
	public $shippingTaxRate = 0.00;
	public $shippingTaxDescription = "";
	public $shippingTaxAmount = 0.00;
	
	public $giftMessage = "";
	
	//payment
	public $paymentIsRealtime = "";
	public $paymentMethodId = "";
	public $paymentMethodName = "";
	public $paymentGatewayId = "";
	
	//buy safe
	public $buysafeWantsBond = "";
	public $buysafeBondAmount = 0.00;
	
	//order total amount
	public $totalAmount = 0;
		
	// *** GIFT CERTIFICATE START
	public $gift_cert_amount = 0;
	// *** GIFT CERTIFICATE END
		
	
	/**
	 * Order Constructor
	 * @param $db
	 * @param $settings
	 * @param $user
	 * @param $need_db_record
	 * @return ORDER
	 */
	public function __construct(&$db, &$settings, &$user, $need_db_record=false)
	{
		$this->_db = $db;
		$this->_settings = $settings;
		$this->_user = $user;
		
		global $_SESSION;
		//////////////////////////
		// set initial params
		$this->errors = array();
		$this->oid = in_array("OrderID", array_keys($_SESSION))?trim($_SESSION["OrderID"]):"";
		
		$this->taxCountryId = $this->_settings["TaxDefaultCountry"];
		$this->taxStateId = $this->_settings["TaxDefaultState"];
		
		///////////////////////////////////////////////////////////
		// trying to restore order by order id or create new one
		//$uptate_user_info = false;
		if ($this->oid != "" && $this->oid > 0)
		{
			$this->db()->query("SELECT uid, oid FROM ".DB_PREFIX."orders WHERE oid='".intval($this->oid)."' AND (status='Abandon' OR status='New') AND (payment_status = 'Pending')");
			if ($this->db()->moveNext())
			{
				//use existing order
				$this->oid = $this->db()->col["oid"];
				if (($this->db()->col["uid"] <> $this->user()->id))
				{
					$this->db()->reset();
					$this->db()->assign("uid", $this->user()->id);
					$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
				}
				$_fl_create_new_order = false;
			}
			else
			{
				$this->oid = 0;
				$_fl_create_new_order = true;
			}
		}
		else
		{
			$this->oid = 0;
			$_fl_create_new_order = true;
		}
			
		if ($_fl_create_new_order && $need_db_record)
		{
			//create new order if there no orders yet
			//$update_user_info = $this->user()->id > 0;
			$this->db()->reset();
			$this->db()->assign("uid", $this->user()->id);
			$this->db()->assign("create_date", "NOW()");
			$this->db()->assign("status_date", "NOW()");
			$this->db()->assignStr("status", "New");
			$this->oid = $this->db()->insert(DB_PREFIX."orders");
			$_fl_create_new_order = false;
		}
		$_SESSION["OrderID"] = $this->oid;
			
		// *** GIFT CERTIFICATE START
		$gift_cert = new GiftCertificates($this->_db);
		$data = $gift_cert->GetOrderGiftCert($this->oid);
		if ($data['gift_cert_valid'] == '1')
		{
			$this->gift_cert_amount = $data['gift_cert_amount'];
		} 
		else
		{
			$this->gift_cert_amount = 0;
		}
		// *** GIFT CERTIFICATE END
			
		return $this;
	}
				
	/**
	 * Set order error message
	 * @param $str
	 */
	public function setError($message)
	{
		$this->errors[count($this->errors)] = $message;
		$this->is_error = true;
	}
	
	/**
	 * Database class instance
	 * @return DB
	 */
	public function db()
	{
		return $this->_db;
	}
	
	/**
	 * Returns class instance
	 * @return USER
	 */
	public function user()
	{
		return $this->_user;
	}
	
	/**
	 * Create new cart from existing one
	 * @param $uid
	 */
	public function newCart($uid)
	{
		$this->user()->id = $uid;
			
		$this->db()->reset();
		$this->db()->assign("uid", intval($this->user()->id));
		$this->db()->assign("create_date", "NOW()");
		$this->db()->assign("status_date", "NOW()");
		$this->db()->assignStr("status", "New");
		$this->oid = $this->db()->insert(DB_PREFIX."orders");
			
		$_SESSION["OrderID"] = $this->oid;
	}
	
	/**
	 * Set abaondon status 
	 */
	public function setAbandonStatus()
	{
		$this->db()->reset();
		$this->db()->assignStr("status", "Abandon");
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
	}
	
	/**
	 * Resets promo code
	 */
	public function resetPromo()
	{
		$this->promoDiscountAmount = 0;
		$this->promoDiscountValue = 0;
		$this->promoDiscountType = "";
		$this->db()->reset();
		$this->db()->assignStr("promo_campaign_id", 0);
		$this->db()->assignStr("promo_discount_amount", 0);
		$this->db()->assignStr("promo_discount_value", 0);
		$this->db()->assignStr("promo_discount_type", "none");
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
	}
	
	/**
	 * Checks is promo code correct
	 * @param $promo_code
	 * @param $subtotal
	 */
	public function checkPromoCode($promo_code)
	{
		/**
		 * Recalc subtotal here again - but skip gift certificates - issue 770
		 */
		$subtotal = 0;
		$this->db()->query("SELECT SUM(quantity * price) AS subtotal_amount FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND product_id <> 'gift_certificate'");
		if ($this->db()->moveNext()) $subtotal = round($this->db()->col["subtotal_amount"], 2);
		
		if (validate($promo_code, VALIDATE_STRING) && $this->_settings["DiscountsPromo"] == "YES")
		{
			//select promo code
			$this->db()->query("SELECT * FROM ".DB_PREFIX."promo_codes WHERE date_start < NOW() AND date_stop > NOW() AND promo_code='".$this->db()->escape($promo_code)."' AND is_active='Yes' AND min_amount <= ".$subtotal);

			if ($this->db()->moveNext())
			{
				$promo = $this->db()->col;
				
				if ($promo["is_global"] == "No")
				{
					$product = false;
					//check does this promo applicable for current order - is PRODUCT attached to promo
					$this->db()->query("
						SELECT ".DB_PREFIX."orders_content.* 
						FROM ".DB_PREFIX."orders_content
						INNER JOIN ".DB_PREFIX."promo_products ON ".DB_PREFIX."promo_products.pid = '".$promo["pid"]."' AND ".DB_PREFIX."promo_products.prid = orders_content.pid
						WHERE ".DB_PREFIX."orders_content.oid='".$this->oid."'
					");
					
					if ($this->db()->numRows() > 0)
					{
						$product = $this->db()->moveNext();
					}
					
					if (!$product)
					{
						//check does this promo applicable for current order - is PRODUCT CATEGORY attached to promo
						$this->db()->query("
							SELECT ".DB_PREFIX."orders_content.* 
							FROM ".DB_PREFIX."orders_content
							INNER JOIN ".DB_PREFIX."products_categories ON ".DB_PREFIX."products_categories.pid = ".DB_PREFIX."orders_content.pid  
							INNER JOIN ".DB_PREFIX."promo_categories ON ".DB_PREFIX."promo_categories.pid = '".$promo["pid"]."' AND ".DB_PREFIX."promo_categories.cid = ".DB_PREFIX."products_categories.cid
							WHERE ".DB_PREFIX."orders_content.oid='".$this->oid."'
						");
						if ($this->db()->numRows() > 0)
						{
							$product = $this->db()->moveNext();
						}
					}
					
					if ($product)
					{
						if ($promo["discount_type"] == "amount")
							$promo_discount_amount = round($promo["discount"] * $product["quantity"], 2);
						else
							$promo_discount_amount = round($product["price"] * $product["quantity"] / 100 * $promo["discount"], 2);
						
						$this->db()->reset();
						$this->db()->assignStr("promo_discount_amount", $promo_discount_amount); 
						$this->db()->assignStr("promo_discount_value", $promo_discount_amount); //set to total discount on product level
						$this->db()->assignStr("promo_campaign_id", $promo["pid"]);
						$this->db()->assignStr("promo_discount_type", "amount"); //on product level it is eq. to amount
						$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
						return true;
					}
				}
				else
				{
					if ($promo["discount_type"] == "amount")
						$promo_discount_amount = round($promo["discount"], 2);
					else
						$promo_discount_amount = round($subtotal / 100 * $promo["discount"], 2);
						
					$this->db()->reset();	
					$this->db()->assignStr("promo_discount_amount", $promo_discount_amount);
					$this->db()->assignStr("promo_discount_value", $promo["discount"]);
					$this->db()->assignStr("promo_campaign_id", $promo["pid"]);
					$this->db()->assignStr("promo_discount_type", $promo["discount_type"]);
					$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
					return true;
				}
			}
		}
		$this->resetPromo();
	
		return false;
	}
		
	/**
	 * Add a new item to a cart
	 * @param $product_id
	 * @param $quantity
	 * @param $attributes
	 * @return boolean
	 */
	public function addItem($product_id, $quantity, $attributes)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."products WHERE product_id='".$this->db()->escape($product_id)."' AND is_visible='Yes' AND call_for_price='No'");
		if ($this->db()->moveNext())
		{
			$product = $this->db()->col;
			$pid = $product["pid"];
								
			//set new tax rate based on region 
			if ($product["is_taxable"] == "Yes")
			{
				$_tax_rate = 0;
				$_tax_description = "";
				if ($product["tax_rate"] > 0)
				{
					$_tax_rate = $product["tax_rate"];
					$_tax_description = "";
				}
				elseif ($product["tax_class_id"] > 0 && array_key_exists($product["tax_class_id"], $this->taxRates))
				{
					$_tax_rate = $this->taxRates[$product["tax_class_id"]]["tax_rate"];
					$_tax_description = $this->taxRates[$product["tax_class_id"]]["description"];
				}
				else
				{
					$product["is_taxable"] = "No";
					$_tax_rate = 0;
					$_tax_description = "";
				}
				$product["tax_rate"] = $_tax_rate;
				$product["tax_description"] = $_tax_description;
			}
			else
			{
				$product["tax_rate"] = 0;
				$product["tax_description"] = "";
			}
				
			$product["final_price"]  = $product["price"];
				
			if ($this->_settings["WholesaleDiscountType"] == "PRODUCT")
			{
				if($this->user()->level > 0 && $product["price_level_1"] > 0 && $this->_settings["WholesaleMaxLevels"] > 0) $product["final_price"] = $product["price_level_1"];
				if($this->user()->level > 1 && $product["price_level_2"] > 0 && $this->_settings["WholesaleMaxLevels"] > 1) $product["final_price"] = $product["price_level_2"];
				if($this->user()->level > 2 && $product["price_level_3"] > 0 && $this->_settings["WholesaleMaxLevels"] > 2) $product["final_price"] = $product["price_level_3"];
			}
				
			//calc price changes based on attributes
			//and store attributes
			$selected_options = "";
			$selected_options_clean = "";
			$this->db()->query("
				SELECT * FROM ".DB_PREFIX."products_attributes
				WHERE pid = '".$pid."' AND is_active = 'Yes'
				ORDER BY priority, name, is_modifier
			");
				
			$mod_price = $product["final_price"];
			$mod_weight = $product["weight"];
			$selected_attributes = array();
			$intentory_list = "";
			while ($this->db()->moveNext())
			{
				$options = array();
				$selected_attribute = array();
				$selected_attribute["attribute_value"] = 0;
				$selected_attribute["attribute_value_type"] = "amount";
					
				if ($this->db()->col["attribute_type"] == "select" || $this->db()->col["attribute_type"] == "radio")
				{
					if (parseOptions($this->db()->col["options"], $product["final_price"], $product["weight"], $options))
					{
						if (array_key_exists($this->db()->col["paid"], $attributes))
						{
							$selected_option = urldecode($attributes[$this->db()->col["paid"]]);
							$selected_options.= $this->db()->col["name"]." : ".$selected_option;
							$selected_options.= (($this->db()->col["is_modifier"]=="Yes")?$options[$selected_option]["modifier"]:"")."\r\n";
							
							if ($this->db()->col["is_modifier"]=="Yes")
							{
								$selected_options_clean.= $this->db()->col["name"]." : ".$options[$selected_option]["full"]."\r\n";
							}
							else
							{
								$selected_options_clean.= $this->db()->col["name"]." : ".$selected_option."\r\n";
							}								
								
							if ($this->db()->col["is_modifier"]=="Yes")
							{
								$mod_price = $mod_price + $options[$selected_option]["price_difference"];
								$selected_attribute["attribute_value"] = $options[$selected_option]["option_value"];
								$selected_attribute["attribute_value_type"] = $options[$selected_option]["option_value_type"];
								$mod_weight = $mod_weight + $options[$selected_option]["weight_difference"];
							}
								
							if (($product["inventory_control"] == "AttrRuleInc" || $product["inventory_control"] == "AttrRuleExc") && $this->db()->col["track_inventory"] == "1")
							{								
								$intentory_list = $intentory_list.($intentory_list == "" ? "" : "\n").$this->db()->col["paid"].": ".$selected_option;
							}
						}
					}
				}
				else
				{
					$selected_option = $attributes[$this->db()->col["paid"]];
					$selected_options.= $this->db()->col["name"]." : ".$selected_option."\r\n";
					$selected_options_clean.= $this->db()->col["name"]." : ".$selected_option."\r\n";
				}
				$selected_attributes[] = $selected_attribute;
			}
				
			$product["final_price"] = $mod_price;				
				
			if ($this->_settings["WholesaleDiscountType"] == "GLOBAL")
			{
				if($this->user()->level > 0 && $this->_settings["WholesaleMaxLevels"] > 0) $product["final_price"] = $product["final_price"]/100*(100-$this->_settings["WholesaleDiscount1"]);
				if($this->user()->level > 1 && $this->_settings["WholesaleMaxLevels"] > 1) $product["final_price"] = $product["final_price"]/100*(100-$this->_settings["WholesaleDiscount2"]);
				if($this->user()->level > 2 && $this->_settings["WholesaleMaxLevels"] > 2) $product["final_price"] = $product["final_price"]/100*(100-$this->_settings["WholesaleDiscount3"]);
			}
				
			$product["final_price"] = (($product["case_pack"]>0) && ($this->_settings["WholesaleCasePack"] == "YES"))?$product["final_price"]*$product["case_pack"]:$product["final_price"];
				
			$product["weight"] = $mod_weight;

			$selected_options = trim($selected_options);
			$selected_options_clean = trim($selected_options_clean);
				
			//inventory, min / max order
			if ($product["inventory_control"] == "Yes")
			{
				$product["allowed_max"] = $product["max_order"] > 0 ? ($product["max_order"] > $product["stock"] ? $product["stock"] : $product["max_order"]) : $product["stock"];
			}
			else
			{
				$product["allowed_max"] = $product["max_order"];
			}
				
			$inventory_id = 0;
			$product_sub_id = "";
				
			if (($product["inventory_control"] == "AttrRuleInc" || $product["inventory_control"] == "AttrRuleExc") && $intentory_list != "")
			{				
				$this->db()->query("SELECT * FROM ".DB_PREFIX."products_inventory WHERE pid='".$pid."' AND attributes_hash='".md5($intentory_list)."'");
				if ($this->db()->moveNext())
				{
					$product["allowed_max"] = $product["max_order"] > 0 ? ($product["max_order"] > $this->db()->col["stock"] ? $this->db()->col["stock"] : $product["max_order"]) : $this->db()->col["stock"];
					$inventory_id = $this->db()->col["pi_id"];
					$product_sub_id = $this->db()->col["product_subid"];
				}
				//check how many products already in order
				$this->db()->query("SELECT SUM(quantity) AS current_quantity FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND pid='".$pid."' AND is_gift='No' AND options LIKE '".$this->db()->escape($selected_options)."'");
				if ($this->db()->moveNext())
				{
					//product already in order
					$current_quantity = $this->db()->col["current_quantity"];
					if (($product["allowed_max"]>0))
					{
						$new_quantity = $current_quantity + $quantity;
						if ($new_quantity > $product["allowed_max"])
						{
							$quantity = $product["allowed_max"] - $current_quantity;
						}
					}
				}
			}
			else
			{
				//check how many products already in order
				$this->db()->query("SELECT SUM(quantity) AS current_quantity FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND pid='".$pid."' AND is_gift='No'");
				if ($this->db()->moveNext())
				{
					//product already in order
					$current_quantity = $this->db()->col["current_quantity"];
					if (($product["allowed_max"]>0))
					{
						$new_quantity = $current_quantity + $quantity;
						if ($new_quantity > $product["allowed_max"])
						{	
							$quantity = $product["allowed_max"] - $current_quantity;
						}
					}
				}
			}
			
			if ($quantity > 0)
			{
			//check is there product in cart with specified id and attributes
				$this->db()->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND pid='".$pid."' AND is_gift= 'No' AND options LIKE '".$this->db()->escape($selected_options)."'");
				if ($this->db()->moveNext())
				{
					//product with id and selected options exists in database
					$ocid = $this->db()->col["ocid"];
					$quantity = $this->db()->col["quantity"] + $quantity;
					$quantity = (($product["min_order"]>0) && ($quantity<$product["min_order"]))?$product["min_order"]:$quantity;
					$quantity = (($product["allowed_max"]>0) && ($quantity>$product["allowed_max"]))?$product["allowed_max"]:$quantity;

					//check quantity discount
					$this->db()->query("SELECT * FROM ".DB_PREFIX."products_quantity_discounts
						WHERE pid='".$pid."' AND ".$quantity." BETWEEN range_min AND range_max
					");

					if ($this->db()->moveNext())
					{
						if (($this->user()->level < 1) || ($this->user()->level > 0 && $this->db()->col["apply_to_wholesale"] == "Yes"))
						{
							if ($this->db()->col["discount_type"] == "amount")
							{
								$product["final_price"] = $product["final_price"] - $this->db()->col["discount"];
							}
							else
							{
								$product["final_price"] = $product["final_price"] - ($product["final_price"] / 100 * $this->db()->col["discount"]);
							}
							if ($this->db()->col["free_shipping"] == "Yes")
							{
								$product["free_shipping"] = "Yes";
							}
						}
					}

					//update database
					$this->db()->reset();
					$this->db()->assign("price", round($product["final_price"], 2));
					$this->db()->assign("admin_price", round($product["final_price"], 2));
					$this->db()->assign("quantity", $quantity);
					$this->db()->assign("cost", $product["cost"]);
					$this->db()->assign("admin_quantity", $quantity);
					$this->db()->assign("weight", $product["weight"]);
					$this->db()->assignStr("is_dollar_days", $product["is_dollar_days"]);
					$this->db()->assignStr("is_doba", $product["is_doba"]);
					$this->db()->assignStr("digital_product", $product["digital_product"]);
					$this->db()->assignStr("digital_product_file", $product["digital_product_file"]);
					$this->db()->assignStr("digital_product_key", $product["digital_product"] == "Yes" ? md5(uniqid(rand(),1)) : "");
					$this->db()->assignStr("digital_product_downloads", "0");

					//$this->db()->assign("attribute_value", $attribute_value);
					//$this->db()->assignStr("attribute_value_type", $attribute_value_type);
					$this->db()->assignStr("free_shipping", $product["free_shipping"]);
					$this->db()->assignStr("is_taxable", $product["is_taxable"]);
					$this->db()->assignStr("tax_rate", $product["tax_rate"]);
					$this->db()->assignStr("tax_description", $product["tax_description"]);
					$this->db()->assignStr("product_id", $product["product_id"]);
					$this->db()->assignStr("product_sku", $product["product_sku"]);
					$this->db()->assignStr("product_upc", $product["product_upc"]);
					$this->db()->assignStr("title", $product["title"]);
					$this->db()->assignStr("options", $selected_options);
					$this->db()->assignStr("options_clean", $selected_options_clean);
					$this->db()->assignStr("product_sub_id", $product_sub_id);
					$this->db()->assignStr("inventory_id", $inventory_id);
					$this->db()->assignStr("products_location_id", $product["products_location_id"]);
					$this->db()->update(DB_PREFIX."orders_content", "WHERE ocid='".$ocid."' AND oid='".$this->oid."' AND pid='".$pid."'");
					$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content_attributes WHERE ocid='".$ocid."'");
				}
				else
				{
					//new product for current cart
					$quantity = (($product["min_order"]>0) && ($quantity<$product["min_order"]))?$product["min_order"]:$quantity;
					$quantity = (($product["allowed_max"]>0) && ($quantity>$product["allowed_max"]))?$product["allowed_max"]:$quantity;

					//check quantity discount
					$this->db()->query("SELECT * FROM ".DB_PREFIX."products_quantity_discounts
						WHERE pid='".$pid."' AND ".$quantity." BETWEEN range_min AND range_max
					");

					if ($this->db()->moveNext())
					{
						if (($this->user()->level < 1) || ($this->user()->level > 0 && $this->db()->col["apply_to_wholesale"] == "Yes"))
						{
							if ($this->db()->col["discount_type"] == "amount")
							{
								$product["final_price"] = $product["final_price"] - $this->db()->col["discount"];
							}
							else
							{
								$product["final_price"] = $product["final_price"] - ($product["final_price"] / 100 * $this->db()->col["discount"]);
							}
							if ($this->db()->col["free_shipping"] == "Yes")
							{
								$product["free_shipping"] = "Yes";
							}
						}
					}

					$this->db()->reset();
					$this->db()->assign("oid", $this->oid);
					$this->db()->assign("pid", $pid);
					$this->db()->assign("price", round($product["final_price"], 2));
					$this->db()->assign("admin_price", round($product["final_price"], 2));
					$this->db()->assign("quantity", $quantity);
					$this->db()->assign("cost", $product["cost"]);
					$this->db()->assign("admin_quantity", $quantity);
					$this->db()->assign("weight", $product["weight"]);

					$this->db()->assignStr("is_dollar_days", $product["is_dollar_days"]);
					$this->db()->assignStr("is_doba", $product["is_doba"]);
					$this->db()->assignStr("digital_product", $product["digital_product"]);
					$this->db()->assignStr("digital_product_file", $product["digital_product_file"]);
					$this->db()->assignStr("digital_product_key", $product["digital_product"] == "Yes" ? md5(uniqid(rand(),1)) : "");
					$this->db()->assignStr("digital_product_downloads", "0");

					//$this->db()->assign("attribute_value", $attribute_value);
					//$this->db()->assignStr("attribute_value_type", $attribute_value_type);
					$this->db()->assignStr("free_shipping", $product["free_shipping"]);
					$this->db()->assignStr("is_taxable", $product["is_taxable"]);
					$this->db()->assignStr("tax_rate", $product["tax_rate"]);
					$this->db()->assignStr("tax_description", $product["tax_description"]);
					$this->db()->assignStr("product_id", $product["product_id"]);
					$this->db()->assignStr("product_sku", $product["product_sku"]);
					$this->db()->assignStr("product_upc", $product["product_upc"]);
					$this->db()->assignStr("title", $product["title"]);
					$this->db()->assignStr("options", $selected_options);
					$this->db()->assignStr("options_clean", $selected_options_clean);
					$this->db()->assignStr("product_sub_id", $product_sub_id);
					$this->db()->assignStr("inventory_id", $inventory_id);
					$this->db()->assignStr("products_location_id", $product["products_location_id"]);
					$ocid = $this->db()->insert(DB_PREFIX."orders_content");
				}
					
				if (count($selected_attributes) > 0)
				{
					for ($i=0; $i<count($selected_attributes); $i++)
					{
						if ($selected_attributes[$i]["attribute_value"] * 1 != 0)
						{
							$this->db()->reset();
							$this->db()->assignStr("ocid", $ocid);
							$this->db()->assignStr("oid", $this->oid);
							$this->db()->assignStr("attribute_value", $selected_attributes[$i]["attribute_value"]);
							$this->db()->assignStr("attribute_value_type", $selected_attributes[$i]["attribute_value_type"]);
							$this->db()->insert(DB_PREFIX."orders_content_attributes");
						}
					}
				}
				return true;
			}
		}
		else
		{
			$this->setError("Incorrect product ID");
		}
		return false;
	}
		
	/**
	 * Remove item from a cart
	 * @param $ocid
	 * @return boolean
	 */
	public function removeItem($ocid)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE ocid='".intval($ocid)."' AND oid='".$this->oid."' AND is_gift = 'No'");
		if ($this->db()->moveNext())
		{
			$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content_attributes WHERE ocid='".intval($ocid)."'");
			$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content WHERE ocid='".intval($ocid)."' AND oid='".$this->oid."'");
			return true;
		}
		else
		{
			$this->setError("Incorrect product ID");
			return false;
		}
	}
		
	/**
	 * Update free products in a cart
	 */
	public function checkFreeProducts()
	{
		$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND is_gift='Yes'");
		$this->db()->query("
			SELECT ".DB_PREFIX."orders_content.pid, SUM(".DB_PREFIX."orders_content.quantity) AS sq, ".DB_PREFIX."products.gift_quantity
			FROM ".DB_PREFIX."orders_content 
			LEFT JOIN ".DB_PREFIX."products ON ".DB_PREFIX."products.pid = ".DB_PREFIX."orders_content.pid
			WHERE oid='".$this->oid."'
			GROUP BY ".DB_PREFIX."orders_content.pid
			ORDER BY ".DB_PREFIX."orders_content.pid
		");
		if ($this->db()->numRows() > 0)
		{
			$order_items = $this->db()->getRecords();
			for ($i=0; $i<count($order_items); $i++)
			{
				$order_item = $order_items[$i];
				
				if ($order_item["gift_quantity"] > 0 && $order_item["sq"] >= $order_item["gift_quantity"])
				{
					//if($order_item["sq"] >= $order_item["gift_quantity"]){
					// $order_item["gift_quantity"];
					$mult = floor($order_item["sq"] / $order_item["gift_quantity"]);
					$this->db()->query("
					    SELECT 
							".DB_PREFIX."products_gifts.gift_quantity AS g_q,
							".DB_PREFIX."products_gifts.gift_shipping_free AS g_s_f,
							".DB_PREFIX."products_gifts.gift_max_quantity AS g_m_q,
							".DB_PREFIX."products.*
						FROM ".DB_PREFIX."products_gifts 
						INNER JOIN ".DB_PREFIX."products ON ".DB_PREFIX."products.pid = ".DB_PREFIX."products_gifts.gift_pid
						WHERE ".DB_PREFIX."products_gifts.pid='".$order_item["pid"]."' AND ".DB_PREFIX."products.is_visible='Yes'
					");
					if ($this->db()->numRows() > 0)
					{
						$gifts = $this->db()->getRecords();
						for ($j=0; $j<count($gifts); $j++)
						{
							$product = $gifts[$j];
							$quantity = $product["g_q"] * $mult;
							if($quantity > $product["g_m_q"]){ $quantity=$product["g_m_q"]; }
							$this->db()->reset();
							$this->db()->assign("oid", $this->oid);
							$this->db()->assign("pid", $product["pid"]);
							$this->db()->assign("price", 0);
							$this->db()->assign("admin_price", 0);
							$this->db()->assign("quantity", $quantity);
							$this->db()->assign("cost", $product["cost"]);
							$this->db()->assign("admin_quantity", $quantity);
							$this->db()->assign("weight", $product["weight"]);
							
							$this->db()->assignStr("free_shipping", $product["g_s_f"]);
							$this->db()->assignStr("product_id", $product["product_id"]);
							$this->db()->assignStr("product_sku", $product["product_sku"]);
							$this->db()->assignStr("product_upc", $product["product_upc"]);
							$this->db()->assignStr("title", $product["title"]);
							$this->db()->assignStr("options", "");
							$this->db()->assignStr("is_gift", "Yes");
							$this->db()->assignStr("digital_product", $product["digital_product"]);
							$this->db()->assignStr("digital_product_file", $product["digital_product_file"]);
							$this->db()->assignStr("digital_product_key", $product["digital_product"] == "Yes" ? md5(uniqid(rand(),1)) : "");
							$this->db()->assignStr("digital_product_downloads", "0");
							$this->db()->assignStr("products_location_id", $product["products_location_id"]);
							$this->db()->insert(DB_PREFIX."orders_content");
						}
					}
				}
			}
		}
	}
		
	/**
	 * Remove all items from a cart
	 */
	public function clearItems()
	{
		$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content_attributes WHERE oid='".$this->oid."'");
		$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		return true;
	}
		
	/**
	 * Stock control
	 * at this point inventory control based on 2 "after checkout" conditions
	 * 1) stock_warning should be < then stock
	 * 2) min_order should be <= then stock
	 */
	public function stockControl()
	{
		// select orders products with allowed stock control from database
		$this->db()->query("
			SELECT ".DB_PREFIX."orders_content.*,
				".DB_PREFIX."products.stock,
				".DB_PREFIX."products.stock_warning,
				".DB_PREFIX."products.min_order,
				".DB_PREFIX."products.max_order,
				".DB_PREFIX."products.inventory_rule
			FROM ".DB_PREFIX."orders_content 
			LEFT JOIN ".DB_PREFIX."products ON 
				".DB_PREFIX."products.pid = ".DB_PREFIX."orders_content.pid AND
				(".DB_PREFIX."products.inventory_control = 'Yes' OR ".DB_PREFIX."products.inventory_control = 'AttrRuleExc' OR ".DB_PREFIX."products.inventory_control = 'AttrRuleInc')
			WHERE 
				".DB_PREFIX."orders_content.oid='".$this->oid."'
		");
		
		// store products in temporary array
		$products = array();
		while ($this->db()->moveNext())
		{
			$products[$this->db()->col["ocid"]] = array(
				"stock"=>$this->db()->col["stock"], 
				"stock_warning"=>$this->db()->col["stock_warning"],
				"pid"=>$this->db()->col["pid"],
				"quantity"=>$this->db()->col["quantity"],
				"min_order"=>$this->db()->col["min_order"],
				"inventory_id"=>$this->db()->col["inventory_id"],
				"inventory_rule"=>$this->db()->col["inventory_rule"]
			);
		}
		// update stock for products
		reset($products);
		foreach ($products as $ocid=>$product)
		{
			$inventory_id = $product["inventory_id"];
			if ($inventory_id > 0)
			{
				//////////////////////////////////////////////////
				// stock on attribute level
				$this->db()->query("SELECT * FROM ".DB_PREFIX."products_inventory WHERE pi_id='".$inventory_id."'");
				if ($this->db()->moveNext())
				{
					$new_attr_stock = $this->db()->col["stock"] - $product["quantity"];
					$new_attr_stock = $new_attr_stock < 0 ? 0 : $new_attr_stock; 
					if ($new_attr_stock <= $this->db()->col["stock_warning"])
					{
						//$this->db()->assignStr("is_active", "0");
						emailOutOfStock($this->_db, $product["pid"], $new_attr_stock, $inventory_id);
					}						
					$this->db()->reset();							
					$this->db()->assignStr("stock", $new_attr_stock);
					$this->db()->update(DB_PREFIX."products_inventory", "WHERE pid='".$product["pid"]."' AND pi_id='".$inventory_id."'");
					$this->db()->reset();
					$this->db()->query("UPDATE ".DB_PREFIX."products SET stock = stock-".$product["quantity"]." WHERE pid='".$product["pid"]."' ");
				}
			}
			else
			{
				//////////////////////////////////////////////////
				// stock on product level
				$new_stock = $product["stock"] - $product["quantity"];
				$new_stock = $new_stock < 0 ? 0 : $new_stock;
				$this->db()->reset();
				$this->db()->assign("stock", $new_stock);
				$this->db()->update(DB_PREFIX."products", "WHERE pid = '".$product["pid"]."'");
				if (($new_stock <= $product["stock_warning"]) || (($product["min_order"]>0) && ($new_stock < $product["min_order"])))
				{	
					emailOutOfStock($this->_db, $product["pid"], $new_stock, 0);
				}
			}
			$this->db()->reset();
			$this->db()->assignStr("is_stock_changed", "Yes");
			$this->db()->update(DB_PREFIX."orders_content", "WHERE ocid='".$ocid."'");
		}
	}
	
	/**
	 * Checkout attempts
	 * @param $limit
	 * @return boolean
	 */
	public function attempt($limit)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."orders WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			$a = $this->db()->col["attempts"];
			if ($a >= $limit)
			{
				$this->db()->reset();
				$this->db()->assignStr("status", "Failed");
				$this->db()->assignStr("payment_is_realtime", "Yes");
				$this->db()->assignStr("payment_status", "Error");
				$this->db()->assign("status_date", "NOW()");
				$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
				return false;
			}
			else
			{
				$this->db()->reset();
				$this->db()->assign("attempts", "attempts + 1");
				$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
				return true;
			}
		}
		return true;
	}
	
	/**
	 * Sets order number
	 * @return mixed
	 */
	public function setOrderNum()
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."orders WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			if ($this->db()->col["order_num"] < 1)
			{
				$this->db()->query("SELECT MAX(order_num) AS max_order_num FROM ".DB_PREFIX."orders");
				$this->db()->moveNext();
				$order_num = $this->db()->col["max_order_num"] + 1;
				
				if($order_num < $this->_settings["MinOrderNumber"]){
					$order_num = $this->_settings["MinOrderNumber"];
				}
				
				$this->db()->reset();
				$this->db()->assign("order_num", $order_num);
				$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
				$this->order_num = $order_num;
				return $order_num;
			}
			else
			{
				$this->order_num = $this->db()->col["order_num"];
				return $this->order_num;
			}
		}
		return false;
	}
	
	/**
	 * Closes order on user side
	 * @param $order_status
	 * @param $payment_status
	 * @return int
	 */
	public function closeOrder($order_status, $payment_status)
	{
		global $_SESSION;
		$order_num = $this->setOrderNum();
		$this->db()->reset();
		$this->db()->assignStr("status", $order_status);
		$this->db()->assignStr("payment_status", $payment_status);
		$this->db()->assign("create_date", "NOW()");
		$this->db()->assign("status_date", "NOW()");
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		$_SESSION["LastOrderID"] = $this->oid;
		return $order_num;
	}
	
	/**
	 * Checks is shipping is digital
	 * @return boolean
	 */
	public function checkShippingDigital()
	{
		$this->db()->query("SELECT count(*) AS c FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		$this->db()->moveNext();
		$items_count = $this->db()->col["c"];
		if ($items_count > 0)
		{
			$this->db()->query("SELECT count(*) AS c FROM ".DB_PREFIX."orders_content WHERE digital_product='No' AND oid='".$this->oid."'");
			$this->db()->moveNext();
			$this->shippingDigital = $this->db()->col["c"] > 0 ? 0 : 1;
			$tax_data = false;
			$this->db()->reset();
			if ($this->shippingDigital)
			{
				$this->db()->assignStr("shipping_carrier_id", "custom");
				$this->db()->assignStr("shipping_method_id", "digital");
				$this->db()->assignStr("shipping_cm_name", "Digital Download");
				$this->db()->assignStr("shipping_taxable", 0);
				$this->db()->assignStr("shipping_tax_rate", 0);
				$this->db()->assignStr("shipping_tax_description", "");
				$this->db()->assignStr("shipping_tax_amount", "0.00");
			}
		}
		else
		{
			$this->db()->reset();
			$this->shippingDigital = 0;
		}
		$this->db()->assignStr("shipping_digital", $this->shippingDigital);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		return $this->shippingDigital;
	}
		
	/**
	 * Set shipping check status to not completed
	 */
	public function openShipping()
	{
		$this->db()->reset();
		$this->db()->assignStr("is_shipping_price", "No");
		$this->db()->assignStr("shipping_price", "0.00");
		$this->db()->update(DB_PREFIX."orders_content", "WHERE oid='".$this->oid."'");
			
		$this->db()->reset();
			
		//tax-related
		$this->shippingTaxable = $this->_settings["ShippingTaxable"];
		$this->shippingTaxClassId = $this->_settings["ShippingTaxClassId"];
			
		//get tax rate
		if ($this->shippingTaxable && array_key_exists($this->shippingTaxClassId, $this->taxRates))
		{
			$this->shippingTaxRate = $this->taxRates[$this->shippingTaxClassId]["tax_rate"];
			$this->shippingTaxDescription = $this->taxRates[$this->shippingTaxClassId]["description"];
		}
		else
		{
			$this->shippingTaxRate = 0;
			$this->shippingTaxDescription = "";
		}

		$this->db()->shippingCompleted = 0;
		$this->db()->assignStr("shipping_completed", "0");
		$this->db()->assignStr("shipping_ssid", "");
		$this->db()->assignStr("shipping_carrier_id", "");
		$this->db()->assignStr("shipping_method_id", "");
		$this->db()->assignStr("shipping_cm_name", "");
		$this->db()->assignStr("shipping_amount", "0.00");
		$this->db()->assignStr("shipping_taxable", $this->shippingTaxable);
		$this->db()->assignStr("shipping_tax_rate", $this->shippingTaxRate);
		$this->db()->assignStr("shipping_tax_description", $this->shippingTaxDescription);
		$this->db()->assignStr("shipping_tax_amount", "0.00");
		
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
	}
	
	/**
	 * Closes shipping check on user site
	 */
	public function closeShipping()
	{
		$this->shippingCompleted = 1;
		$this->db()->reset();
		$this->db()->assignStr("shipping_completed", "1");
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
	}
		
	/**
	 * Update shipping methods
	 * @param $method_id
	 */
	function updateShippingMethod($method_id)
	{
		//check free shipping
		if($this->_settings["ShippingCalcEnabled"] == "YES")
		{
			if(!$this->isShippingFree())
			{
				//check does method exists
				if($method_id == 0 && $this->_settings["ShippingWithoutMethod"] == "YES"){
					$this->shippingSSID = "0";
					$this->shippingCarrierID = "custom";
					$this->shippingMethodID = "future_charge";
					$this->shippingCMName = "Undefined";
				}
				else
				{
					$this->db()->query("SELECT * FROM ".DB_PREFIX."shipping_selected WHERE ssid='".$method_id."'");
					if ($this->db()->moveNext())
					{
						$this->shippingSSID = $this->db()->col["ssid"];
						$this->shippingCarrierID = $this->db()->col["carrier_id"];
						$this->shippingMethodID = $this->db()->col["method_id"];
						$this->shippingCMName = $this->db()->col["carrier_name"].($this->db()->col["carrier_id"]!="custom"?(" - ".$this->db()->col["method_name"]):"");
					}
					else
					{
						$this->shippingSSID = "0";
						$this->shippingCarrierID = "custom";
						$this->shippingMethodID = "disabled";
						$this->shippingCMName = "Shipping is disabled";
					}
				}
			}
			else
			{
				$this->shippingSSID = "0";
				$this->shippingCarrierID = "custom";
				$this->shippingMethodID = "free";
				$this->shippingCMName = "Shipping is free";
			}
		}
		else
		{
			$this->shippingSSID = "0";
			$this->shippingCarrierID = "custom";
			$this->shippingMethodID = "global_disabled";
			$this->shippingCMName = "Shipping is disabled";
		}
			
		$this->db()->reset();
		$this->db()->assignStr("shipping_amount", "0.00");
		$this->db()->assignStr("shipping_ssid", $this->shippingSSID);
		$this->db()->assignStr("shipping_carrier_id", $this->shippingCarrierID);
		$this->db()->assignStr("shipping_method_id", $this->shippingMethodID);
		$this->db()->assignStr("shipping_cm_name", $this->shippingCMName);
		$this->db()->assignStr("shipping_tax_amount", 0.00);
		$this->db()->assignStr("shipping_tax_rate", 0.00);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		
		return $this->shippingSSID;
	}
		
	/**
	 * Update shipping address information
	 * @param $address
	 */
	public function updateShippingAddress($address)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE stid='".intval($address["state"])."' AND coid='".intval($address["country"])."'");
		if ($this->db()->moveNext())
		{
			$address["province"] = $this->db()->col["name"];
		}
		else
		{
			$address["state"] = 0;	
		}
		
		$this->updateShippingMethod(0);
		$this->db()->reset();
		$this->db()->assignStr("shipping_name", $address["name"]);
		$this->db()->assignStr("shipping_company", $address["company"]);
		$this->db()->assignStr("shipping_address1", $address["address1"]);
		$this->db()->assignStr("shipping_address2", $address["address2"]);
		$this->db()->assignStr("shipping_city", $address["city"]);
		$this->db()->assignStr("shipping_state", intval($address["state"]));
		$this->db()->assignStr("shipping_province", $address["province"]);
		$this->db()->assignStr("shipping_country", intval($address["country"]));
		$this->db()->assignStr("shipping_zip", $address["zip"]);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		if ($this->_settings["TaxAddress"] == "Shipping")
		{
			$this->taxCountryId = $address["country"];
			$this->taxStateId = $address["state"];
			$this->taxRates = $this->getTaxRates();
		}
	}
		
	/**
	 * Update payment method
	 * @param $payment_method_id
	 * @return boolean
	 */
	public function updatePaymentMethod($payment_method_id)
	{
		if ($payment_method_id == 0 || $payment_method_id == "free")
		{
			$this->db()->reset();
			$this->db()->assignStr("payment_is_realtime", "No");
			$this->db()->assignStr("payment_method_id", 0);
			$this->db()->assignStr("payment_method_name", "Free");
			$this->db()->assignStr("payment_gateway_id", "Free");
			$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
			$this->paymentMethodId = 0;
			return true;
		}
		else
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE pid='".intval($payment_method_id)."' AND active='Yes'");
			if ($this->db()->moveNext())
			{
				$this->db()->reset();
				$this->db()->assignStr("payment_is_realtime", $this->db()->col["type"]!="custom"?"Yes":"No");
				$this->db()->assignStr("payment_method_id", $payment_method_id);
				$this->db()->assignStr("payment_method_name", $this->db()->col["name"]);
				$this->db()->assignStr("payment_gateway_id", $this->db()->col["id"]);
				$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
				$this->paymentMethodId = $payment_method_id;
				return true;
			}
			else
			{
				return false;
			}
		}
	}
		
	/**
	 * Update order gift message
	 * @param $gift_message
	 */
	public function storeGiftMessage($gift_message)
	{
		$this->giftMessage = $gift_message;
		$this->db()->query("UPDATE ".DB_PREFIX."orders SET gift_message = '".$this->db()->escape($gift_message)."' WHERE oid='".$this->oid."'");
	}
	
	/**
	 * Run items update again to reset ordered items info
	 */
	public function reupdateItems()
	{
		$this->db()->query("SELECT ocid, quantity FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		$q = array();
		while ($this->db()->moveNext())
		{
			$q[$this->db()->col["ocid"]] = $this->db()->col["quantity"];
		}
		$this->updateItems($q);
	}
		
	/**
	 * Bulk updating of items in the cart
	 * @param $q
	 */
	public function updateItems($q)
	{
		foreach ($q as $ocid=>$quantity)
		{
			//check product
			$ocid *=1;
			$quantity *=1;
			$this->db()->query("
				SELECT 
					".DB_PREFIX."products.*,
					".DB_PREFIX."orders_content.ocid,
					".DB_PREFIX."orders_content.weight as product_weight,
					".DB_PREFIX."orders_content.quantity,
					".DB_PREFIX."orders_content.price AS product_price,
					".DB_PREFIX."orders_content.is_gift,
					".DB_PREFIX."orders_content.inventory_id
				FROM ".DB_PREFIX."orders_content
				INNER JOIN ".DB_PREFIX."products ON ".DB_PREFIX."products.pid = ".DB_PREFIX."orders_content.pid
				WHERE 
					".DB_PREFIX."orders_content.ocid='".$ocid."' AND
					".DB_PREFIX."orders_content.oid='".$this->oid."'
			");
			if ($this->db()->moveNext())
			{
				$product = $this->db()->col;
							
				//set/update tax rate based on region 
				if ($product["is_taxable"] == "Yes")
				{
					$_tax_rate = 0;
					$_tax_description = "";
					
					if ($product["tax_rate"] > 0)
					{
						$_tax_rate = $product["tax_rate"];
						$_tax_description = "";
					}
					elseif ($product["tax_class_id"] && array_key_exists($product["tax_class_id"], $this->taxRates))
					{
						$_tax_rate = $this->taxRates[$product["tax_class_id"]]["tax_rate"];
						$_tax_description = $this->taxRates[$product["tax_class_id"]]["description"];
					}
					else
					{
						$product["is_taxable"] = "No";
						$_tax_rate = 0;
						$_tax_description = "";
					}
					$product["tax_rate"] = $_tax_rate;
					$product["tax_description"] = $_tax_description;
				}
				else
				{
					$product["tax_rate"] = 0;
					$product["tax_description"] = "";
				}
				
				//inventory, min / max order
				if ($product["inventory_control"] == "Yes")
				{
					$product["allowed_max"] = $product["max_order"] > 0 ? ($product["max_order"] > $product["stock"] ? $product["stock"] : $product["max_order"]) : $product["stock"];
				}					
				elseif ($product["inventory_control"] == "AttrRuleInc" || $product["inventory_control"] == "AttrRuleExc")
				{
					$this->db()->query("SELECT * FROM ".DB_PREFIX."products_inventory WHERE pi_id='".$product["inventory_id"]."'");
					if ($this->db()->moveNext())
					{
						$product["allowed_max"] = $product["max_order"] > 0 ? ($product["max_order"] > $this->db()->col["stock"] ? $this->db()->col["stock"] : $product["max_order"]) : $this->db()->col["stock"];
					}
				}
				else
				{
					$product["allowed_max"] = $product["max_order"];
				}
				
				if ($quantity > 0)
				{
					$quantity = ($product["min_order"] > 0) && ($quantity < $product["min_order"])?$product["min_order"]:$quantity;
					$quantity = (isset($product["allowed_max"]) && $product["allowed_max"] > 0) && ($quantity > $product["allowed_max"])?$product["allowed_max"]:$quantity;
					
					$product["final_price"]  = $product["price"];
					if ($this->_settings["WholesaleDiscountType"] == "PRODUCT")
					{
						if ($this->user()->level > 0 && $product["price_level_1"] > 0 && $this->_settings["WholesaleMaxLevels"] > 0) $product["final_price"] = $product["price_level_1"];
						if ($this->user()->level > 1 && $product["price_level_2"] > 0 && $this->_settings["WholesaleMaxLevels"] > 1) $product["final_price"] = $product["price_level_2"];
						if ($this->user()->level > 2 && $product["price_level_3"] > 0 && $this->_settings["WholesaleMaxLevels"] > 2) $product["final_price"] = $product["price_level_3"];
					}					
									
					$mod_price = $product["final_price"];
					
					$this->db()->query("SELECT * FROM ".DB_PREFIX."orders_content_attributes WHERE ocid='".$product["ocid"]."'");
					while ($this->db()->moveNext())
					{
						if ($this->db()->col["attribute_value"] * 1 != 0)
						{
							if ($this->db()->col["attribute_value_type"] == "amount")
							{
								$mod_price = $mod_price + $this->db()->col["attribute_value"];
							}
							else{
								$mod_price = $mod_price + $product["final_price"] / 100 * $this->db()->col["attribute_value"];
							}
						}
					}
					$product["final_price"] = $mod_price;
					
					if ($this->_settings["WholesaleDiscountType"] == "GLOBAL")
					{
						$_f_p = $product["final_price"];
						if ($this->user()->level > 0 && $this->_settings["WholesaleMaxLevels"] > 0) $product["final_price"] = $_f_p/100*(100-$this->_settings["WholesaleDiscount1"]);
						if ($this->user()->level > 1 && $this->_settings["WholesaleMaxLevels"] > 1) $product["final_price"] = $_f_p/100*(100-$this->_settings["WholesaleDiscount2"]);
						if ($this->user()->level > 2 && $this->_settings["WholesaleMaxLevels"] > 2) $product["final_price"] = $_f_p/100*(100-$this->_settings["WholesaleDiscount3"]);
					}
					
					$product["final_price"] = (($product["case_pack"]>0) && ($this->_settings["WholesaleCasePack"] == "YES"))?$product["final_price"]*$product["case_pack"]:$product["final_price"];
						
					//check quantity discount
					$this->db()->query("SELECT * FROM ".DB_PREFIX."products_quantity_discounts
						WHERE pid='".$product["pid"]."' AND ".$quantity." BETWEEN range_min AND range_max
					");
					if ($this->db()->moveNext())
					{
						if (($this->user()->level < 1) || ($this->user()->level > 0 && $this->db()->col["apply_to_wholesale"] == "Yes"))
						{
							if ($this->db()->col["discount_type"] == "amount")
							{
								$product["final_price"] = $product["final_price"] - $this->db()->col["discount"];
							}
							else
							{
								$product["final_price"] = $product["final_price"] - ($product["final_price"] / 100 * $this->db()->col["discount"]);
							}
							if ($this->db()->col["free_shipping"] == "Yes")
							{
								$product["free_shipping"] = "Yes";
							}
						}
					}
					
					// *** GIFT CERTIFICATE
					if ($product["product_id"] != 'gift_certificate')
					{
					// *** GIFT CERTIFICATE
						$this->db()->reset();
						if ($product["is_gift"] == "No")
						{
							$this->db()->assign("price", round($product["final_price"], 2));
							$this->db()->assign("admin_price", round($product["final_price"], 2));
							$this->db()->assignStr("free_shipping", $product["free_shipping"]);
						}
						else
						{
							$this->db()->assign("price", 0);
							$this->db()->assign("admin_price", 0);
						}
						$this->db()->assign("quantity", $quantity);
						$this->db()->assign("cost", $product["cost"]);
						$this->db()->assign("admin_quantity", $quantity);
						$this->db()->assign("weight", $product["product_weight"]);
						$this->db()->assignStr("is_taxable", $product["is_taxable"]);
						$this->db()->assignStr("tax_rate", $product["tax_rate"]);
						$this->db()->assignStr("tax_description", $product["tax_description"]);
						$this->db()->assignStr("product_id", $product["product_id"]);
						$this->db()->assignStr("product_sku", $product["product_sku"]);
						$this->db()->assignStr("product_upc", $product["product_upc"]);
						$this->db()->assignStr("title", $product["title"]);
						$this->db()->update(DB_PREFIX."orders_content", "WHERE oid='".$this->oid."' AND ocid='".$ocid."'");
					// *** GIFT CERTIFICATE
					}
					// *** GIFT CERTIFICATE
				}
				else
				{
					$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND ocid='".$ocid."'");
					$this->db()->query("DELETE FROM ".DB_PREFIX."orders_content_attributes WHERE oid='".$this->oid."' AND ocid='".$ocid."'");
				}
			}
			else
			{
				$this->setError("Can't update item. Incorrect product id #".$ocid);
			}
		}
	}
	
	/**
	 * Calculate items count
	 * @return int
	 */
	public function getItemsCount()
	{
		$this->db()->query("SELECT SUM(quantity) AS items_count FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			$this->itemsCount = $this->db()->col["items_count"];
			$this->itemsCount = $this->itemsCount > 0 ? $this->itemsCount : 0;
		}
		else
		{
			$this->itemsCount = 0;
		}
		return $this->itemsCount;
	}
	
	/**
	 * Check is shipping is free
	 * @return unknown_type
	 */
	public function isShippingFree()
	{
		$this->db()->query("SELECT COUNT(*) AS c FROM ".DB_PREFIX."orders_content WHERE free_shipping='No' AND oid='".$this->oid."'");
		$this->db()->moveNext();
		return !($this->db()->col["c"] > 0);
	}
		
	/**
	 * Calculate total amount
	 * @return numeric
	 */
	public function getTotalAmount()
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."orders WHERE oid='".($this->oid)."'");
		if ($this->db()->moveNext())
		{
			$this->totalAmount = 
				round($this->db()->col["subtotal_amount"], 2) +
				round($this->db()->col["tax_amount"], 2) +
				round($this->db()->col["shipping_amount"], 2) -
				round($this->db()->col["discount_amount"], 2) -
				round($this->db()->col["promo_discount_amount"], 2) +
				($this->db()->col["buysafe_wants_bond"] == "true" ? round($this->db()->col["buysafe_bond_amount"], 2) : 0) + 
				($this->handlingSeparated ? round($this->db()->col["handling_amount"], 2) : 0.00);
			$this->totalAmount = $this->totalAmount > 0 ? $this->totalAmount : 0.00;
			$this->db()->query("UPDATE ".DB_PREFIX."orders SET total_amount='".round($this->totalAmount, 2)."' WHERE oid='".($this->oid)."'");
		}
		return round($this->totalAmount, 2);
	}
	
	
	/**
	 * Calculate total amount
	 * @return numeric
	 */
	public function getCartTotalAmount()
	{
		return $this->getTotalAmount() - (($this->_settings['DisplayPricesWithTax'] == 'NO')
			? $this->getTaxAmount()
			: 0);
	}
	
		
	/**
	 * Calc subtotal and store it into db
	 * @return numeric
	 */
	public function getSubtotalAmount()
	{
		$this->db()->query("SELECT SUM(quantity * price) AS subtotal_amount FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			$this->subtotalAmount = $this->db()->col["subtotal_amount"];
			$this->subtotalAmount = round($this->subtotalAmount > 0 ? $this->subtotalAmount : 0.00, 2);
		}
		else
		{
			$this->subtotalAmount = 0.00;
		}
		
		$this->db()->reset();
		$this->subtotalAmount = round($this->subtotalAmount, 2);
		$this->db()->assign("subtotal_amount", $this->subtotalAmount);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		return $this->subtotalAmount;
	}
		
	/**
	 * Calc promo discount
	 * @param $subtotal
	 * @return numeric
	 */
	public function getPromoDiscountAmount()
	{
		/**
		 * Recalc subtotal here again - but skip gift certificates - issue 770
		 */
		$subtotal = 0;
		$this->db()->query("SELECT SUM(quantity * price) AS subtotal_amount FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND product_id <> 'gift_certificate'");
		if ($this->db()->moveNext()) $subtotal = round($this->db()->col["subtotal_amount"], 2);
		
		$this->promoDiscountAmount = 0;
		$this->db()->query("SELECT * FROM ".DB_PREFIX."orders WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			// Ricky26: Local promotions
			$order = $this->db()->col;
			$this->db()->reset();
			
			if($order["promo_campaign_id"] == 0)
				return 0;
			
			$this->db()->query("SELECT * FROM `".DB_PREFIX."promo_codes` WHERE pid = '".$order["promo_campaign_id"]."';");
			if(!$this->db()->moveNext())
				return 0;
			
			$promo = $this->db()->col;
			if($promo["is_global"] == "No")
			{
				// We have to calculate a different sub total, and make sure the new
				// subtotal is bigger than our minimum amount. -- Ricky26
				$this->db()->query("SELECT DISTINCT o.pid, o.price, o.quantity FROM `".DB_PREFIX."orders_content` o "
				."INNER JOIN `".DB_PREFIX."orders` oz ON o.oid = oz.oid "
				."INNER JOIN `".DB_PREFIX."promo_codes` pro ON pro.pid = oz.promo_campaign_id "
				."INNER JOIN `".DB_PREFIX."products_categories` c ON o.pid = c.pid "
				."LEFT JOIN `".DB_PREFIX."promo_categories` pc ON c.cid = pc.cid AND pc.pid = pro.pid "
				."LEFT JOIN `".DB_PREFIX."promo_products` pp ON o.pid = pp.prid AND pp.pid = pro.pid "
				."WHERE o.oid = '".$this->oid."' AND (pp.pid IS NOT NULL OR pc.pid IS NOT NULL);");

				$totalPrice = 0.0;
				$count = 0;
				while($this->db()->moveNext())
				{
					$totalPrice += $this->db()->col["price"];
					if($this->db()->col["quantity"] > 0)
						$count += $this->db()->col["quantity"];
					else
						$count++;
				}
				
				$this->db()->reset();
				
				if ($promo["discount_type"] == "amount")
					$this->promoDiscountAmount = round($promo["discount"]*$count, 2);
				else
					$this->promoDiscountAmount = round($totalPrice / 100 * $promo["discount"], 2);
					
				$this->promoDiscountValue = $this->promoDiscountAmount;
			}
			else
			{
				
				if ($promo["discount_type"] == "amount")
					$this->promoDiscountAmount = round($promo["discount"], 2);
				else
					$this->promoDiscountAmount = round($subtotal / 100 * $promo["discount"], 2);
				
				$this->promoDiscountValue = $promo["discount"];
					
			}
			// Ricky26: /Local Promotions
		}
		
		$this->db()->reset();
		$this->promoDiscountAmount = round($this->promoDiscountAmount, 2);
		$this->db()->assign("promo_discount_amount", $this->promoDiscountAmount);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		return $this->promoDiscountAmount;
	}
	
	/**
	 * Calc discount amount
	 * @param $subtotal
	 * @return numeric
	 */
	public function getDiscountAmount()
	{
		/**
		 * Recalc subtotal here again - but skip gift certificates - issue 770
		 */
		$subtotal = 0;
		$this->db()->query("SELECT SUM(quantity * price) AS subtotal_amount FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."' AND product_id <> 'gift_certificate'");
		if ($this->db()->moveNext()) $subtotal = round($this->db()->col["subtotal_amount"], 2);
		
		$this->discountAmount = 0;
		$this->discountValue = 0;
		$this->discountType = "none";
		if ($this->user()->level < 1)
		{
			if ($this->_settings["DiscountsActive"] == "YES")
			{
				$this->db()->query("SELECT * FROM ".DB_PREFIX."discounts WHERE active='Yes' AND ".$subtotal.">=min AND ".$subtotal."<=max ORDER BY did");
				if ($this->db()->moveNext())
				{
					$this->discountType = $this->db()->col["type"];
					$this->discountValue = $this->db()->col["discount"];
					switch($this->discountType)
					{
						case "percent" : 
						{
							$this->discountAmount = round($subtotal / 100 * $this->discountValue, 2);
							break;
						}
						case "amount" :
						{
							$this->discountValue = round($this->discountValue, 2);
							$this->discountAmount = $this->discountValue;
							break;
						}
					}
				}
			}
		}
		
		$this->db()->reset();
		$this->db()->assignStr("discount_amount", $this->discountAmount);
		$this->db()->assignStr("discount_value", $this->discountValue);
		$this->db()->assignStr("discount_type", $this->discountType);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		return $this->discountAmount;
	}
	
	/**
	 * Get Handling Amount
	 * @return numeric
	 */
	public function getHandlingAmount(){
		//for digital - no handling
		if ($this->shippingDigital)
		{
			//common
			$this->handlingSeparated = 0;
			$this->handlingFee = 0.00;
			$this->handlingFeeType = "";	
			$this->handlingText = "";
			$this->handlingAmount = 0.00;
			//handling - tax-related
			$this->handlingTaxable = 0; 
			$this->handlingTaxClassId = 0;
			$this->handlingTaxRate = 0.00;
			$this->handlingTaxDescription = "";
			$this->handlingTaxAmount = 0.00;
			$update = true;
		}
		else
		{
			//common
			$this->handlingSeparated = $this->_settings["ShippingHandlingSeparated"];
			$this->handlingFee = $this->_settings["ShippingHandlingFee"];
			$this->handlingFeeType = $this->_settings["ShippingHandlingFeeType"];
			$this->handlingText = $this->_settings["ShippingHandlingText"];
		
			//free shipping
			if ($this->shippingCarrierID == "custom" && $this->shippingMethodID == "free")
			{
				//$this->handlingSeparated = 1;
			}
			
			//get handling amount
			if ($this->handlingFeeType == "percent")
			{
				$this->handlingAmount = round(($this->subtotalAmount - $this->discountAmount - $this->promoDiscountAmount) / 100 * $this->handlingFee, 2);
			}
			else
			{
				$this->handlingAmount = round($this->_settings["ShippingHandlingFee"], 2);
					$this->handlingFee = round($this->_settings["ShippingHandlingFee"], 2);
			}
			
			//tax-related
			$this->handlingTaxable = $this->_settings["ShippingHandlingTaxable"];
			$this->handlingTaxClassId= $this->_settings["ShippingHandlingTaxClassId"];
			
			//get tax rate; if handling is not separated, shipping tax applied
			//if ($this->handlingSeparated && $this->handlingAmount > 0 && $this->handlingTaxable && array_key_exists($this->handlingTaxClassId, $this->taxRates)){
				$this->handlingTaxRate = $this->taxRates[$this->handlingTaxClassId]["tax_rate"];
				$this->handlingTaxDescription = $this->taxRates[$this->handlingTaxClassId]["description"];
				$this->handlingTaxAmount = round($this->handlingAmount / 100 * $this->handlingTaxRate, 2);
			//}
			//else{
				//$this->handlingTaxable = 0;
			//	$this->handlingTaxRate = 0.00;
			//	$this->handlingTaxDescription = "";
			//	$this->handlingTaxAmount = 0.00;
			//	}
			}
		
		//save to database
		$this->db()->reset();
		$this->db()->assignStr("handling_separated", $this->handlingSeparated?"1":"0");
		$this->db()->assignStr("handling_fee", $this->handlingFee);
		$this->db()->assignStr("handling_fee_type", $this->handlingFeeType);
			
		$this->db()->assignStr("handling_text", $this->handlingText);
		$this->db()->assignStr("handling_amount", $this->handlingAmount);
		$this->db()->assignStr("handling_taxable", $this->handlingTaxable?"1":"0");
		$this->db()->assignStr("handling_tax_rate", $this->handlingTaxRate);
		$this->db()->assignStr("handling_tax_amount", $this->handlingTaxAmount);
		$this->db()->assignStr("handling_tax_description", $this->handlingTaxDescription);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		return $this->handlingAmount;
	}
	
	/**
	 * Get Taxes Rates
	 * @return array
	 */
	public function getTaxRates($country = false, $state = false)
	{
		$coid = intval($country ? $country : $this->taxCountryId);
		$stid = intval($state ? $state : $this->taxStateId);
		
	
		
		$query = "
			SELECT ".DB_PREFIX."tax_rates.*
			FROM ".DB_PREFIX."tax_rates
			INNER JOIN ".DB_PREFIX."tax_zones ON (".DB_PREFIX."tax_rates.zone_id = ".DB_PREFIX."tax_zones.zone_id)
			INNER JOIN ".DB_PREFIX."tax_zones_regions ON (".DB_PREFIX."tax_zones_regions.zone_id = ".DB_PREFIX."tax_rates.zone_id)
			WHERE 
				(((".DB_PREFIX."tax_zones_regions.coid = '0') AND (".DB_PREFIX."tax_zones_regions.stid = '0')) OR
				((".DB_PREFIX."tax_zones_regions.coid = '".$coid."') AND (".DB_PREFIX."tax_zones_regions.stid = '0' OR ".DB_PREFIX."tax_zones_regions.stid = '".$stid."')))
				AND user_level  LIKE '%".intval($this->user() != null ? $this->user()->level : 0)."%'
			GROUP BY ".DB_PREFIX."tax_rates.rate_id
			ORDER BY ".DB_PREFIX."tax_rates.class_id, ".DB_PREFIX."tax_rates.rate_priority
		";
		$this->db()->query($query);
		
		$tax_rates = array("0"=>array("multiplier"=>1, "tax_rate"=>0, "description"=>""));
		
		if ($this->db()->numRows() > 0)
		{
			$_tax_rates = $this->db()->getRecords();
			//get summ rates
			for ($i=0; $i<count($_tax_rates); $i++)
			{
				$class_id = $_tax_rates[$i]["class_id"];
				$rate_priority = $_tax_rates[$i]["rate_priority"];
				if (!array_key_exists($class_id, $tax_rates))
				{
					$tax_rates[$class_id] = array(
						"multiplier"=>1,
						"priority"=>array(),
						"description"=>""
					);
				}
				if (!array_key_exists($rate_priority, $tax_rates[$class_id]["priority"]))
				{
					$tax_rates[$class_id]["priority"][$rate_priority] = 0;;
				}
				$tax_rates[$class_id]["priority"][$rate_priority] += $_tax_rates[$i]["tax_rate"];
				$tax_rates[$class_id]["description"].= ($tax_rates[$class_id]["description"] == ""?"":" + ");
				$tax_rates[$class_id]["description"].= $_tax_rates[$i]["rate_description"]." (".myNum($_tax_rates[$i]["tax_rate"])."%)";
			}
			//get compaund rates
			foreach ($tax_rates as $class_id=>$class_data)
			{
				if ($class_id > 0)
				{
					$start = 100;
					foreach ($class_data["priority"] as $rate_priority=>$tax_summ)
					{
						$start = $start + ($start / 100 * $tax_summ);
					}
					$tax_rates[$class_id]["multiplier"] = $start / 100;
					$tax_rates[$class_id]["tax_rate"] = ($tax_rates[$class_id]["multiplier"]-1)*100;
				}
			}
		}
		$this->taxRates = $tax_rates;
		return $tax_rates;
	}
	
	/**
	 * Get Tax Amount
	 * @return numeric
	 */
	public function getTaxAmount()
	{
		$this->reupdateItems();
			
		//calc tax for real paid price, also skip not taxable product
		//determinate discount percent
		$dp = 0;
		if ($this->subtotalAmount > 0)
		{
			if ($this->discountType == "percent")
			{
				$dp = $dp + $this->discountValue;
			}
			else
			{
				$dp = $dp + $this->discountAmount / ($this->subtotalAmount / 100);
			}
		
			if ($this->promoDiscountType == "percent")
			{
				$dp = $dp + $this->promoDiscountValue;
			}
			else
			{
				$dp = $dp + $this->promoDiscountAmount / ($this->subtotalAmount / 100);
			}
		}
		
		
		//$this->db()->query("SELECT SUM(quantity * if (is_taxable='Yes', price / 100 * tax_rate, 0) AS tax_amount FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		$this->db()->query("SELECT SUM(quantity * if (is_taxable='Yes', (price - price / 100 * ".$dp.") * if (tax_rate>0, tax_rate, 0), 0)) / 100 AS tax_amount FROM ".DB_PREFIX."orders_content WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			$this->taxAmount = $this->db()->col["tax_amount"];
			$this->taxAmount = $this->taxAmount > 0 ? $this->taxAmount : 0.00;
		}
		else
		{
			$this->taxAmount = 0.00;
		}
		
		if ($this->shippingTaxable)
		{
			$this->taxAmount = $this->taxAmount + $this->shippingTaxAmount;
		}
		
		if ($this->handlingTaxable)
		{
			$this->taxAmount = $this->taxAmount + $this->handlingTaxAmount;
		}
		
		$this->taxAmount = round($this->taxAmount, 2);
		
		$this->db()->reset();
		$this->db()->assignStr("tax_amount", $this->taxAmount);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
		
		return $this->taxAmount;
	}
	
	/**
	 * get shipping amount based on order data
	 * @param $error_message
	 * @return numeric
	 */
	public function getShippingAmount(&$error_message)
	{
		//////////////////////////////////
		//dollar-days
		$dollar_days_shipping_percent = 15;
		$dollar_days_shipping_min_amount = 6.95;
		
		////////////////////////////////////////
		//select/calc data required for shipping price calculation
		$shipping_price_at_product_level = 0;
		$shipping_amount_for_price_based = 0;
		$shipping_amount_for_price_based_exists = false;
		$shipping_weight = 0;
		$shipping_items_count = 0;
		$shipping_price_dollar_days = 0;
		$shipping_amount_dollar_days = 0;
		$shipping_amount_doba = 0;
		$doba_items = array();
		$ship_user_products = false;
		
		////////////////////////////////////////
		//check is there used product level method
		if (isset($_SESSION["product_level_method"]))
		{
			$product_level_method = $_SESSION["product_level_method"];
			$this->db()->query("
				SELECT 
					".DB_PREFIX."orders_content.*,
					if (".DB_PREFIX."products_shipping_price.is_price IS NULL, 'No', ".DB_PREFIX."products_shipping_price.is_price) AS product_is_shipping_price,
					if (".DB_PREFIX."products_shipping_price.price IS NULL, 0.00, ".DB_PREFIX."products_shipping_price.price) AS product_shipping_price
				FROM ".DB_PREFIX."orders_content 
				LEFT JOIN ".DB_PREFIX."products_shipping_price ON ".DB_PREFIX."products_shipping_price.pid = ".DB_PREFIX."orders_content.pid AND ".DB_PREFIX."products_shipping_price.ssid='".$product_level_method["ssid"]."'
				WHERE ".DB_PREFIX."orders_content.oid='".$this->oid."'
			");
		}
		else
		{
			$product_level_method = false;
			$this->db()->query("
				SELECT 
					".DB_PREFIX."orders_content.*,
					'No' AS product_is_shipping_price,
					'0.00' AS product_shipping_price
				FROM ".DB_PREFIX."orders_content 
				WHERE ".DB_PREFIX."orders_content.oid='".$this->oid."'
			");
		}
		
		$order_content_shipping_price = array();
		
		while ($this->db()->moveNext())
		{
			//dollar days
			if ($this->db()->col["is_dollar_days"] == "Yes")
			{
				$shipping_price_dollar_days = $shipping_price_dollar_days + $this->db()->col["price"] * $this->db()->col["quantity"];
			}
			//doba
			elseif ($this->db()->col["is_doba"] == "Yes")
			{
				$doba_items[] = array("item_id"=>$this->db()->col["product_id"], "quantity"=>$this->db()->col["quantity"]);
			}
			else
			{
				//$order_content_shipping_price[$this->db()->col["ocid"]] = array("is_price"=>"No", "price"=>"0.00000");
				if ($product_level_method && $this->db()->col["product_is_shipping_price"] == "Yes" && $this->db()->col["free_shipping"] == "No" && $this->db()->col["digital_product"] == "No")
				{
					$shipping_price_at_product_level = $shipping_price_at_product_level + round($this->db()->col["product_shipping_price"],2) * $this->db()->col["quantity"];
					$order_content_shipping_price[$this->db()->col["ocid"]] = array("is_shipping_price"=>"Yes", "shipping_price"=>round($this->db()->col["product_shipping_price"], 2));
				}
				if ($this->db()->col["product_is_shipping_price"] == "No" && $this->db()->col["free_shipping"] == "No" && $this->db()->col["digital_product"] == "No")
				{
					$shipping_items_count = $shipping_items_count + $this->db()->col["quantity"];
					$shipping_weight = $shipping_weight + $this->db()->col["weight"] * $this->db()->col["quantity"];
					$shipping_amount_for_price_based = $shipping_amount_for_price_based + $this->db()->col["price"] * $this->db()->col["quantity"];
					$shipping_amount_for_price_based_exists = true;
				}
			}
		}
		
		
		////////////////////////////////////////
		//prepare from(origin) shipping address
		$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE coid='".$this->_settings["ShippingOriginCountry"]."'");
		if ($this->db()->moveNext())
		{
			$address_from = array("country" => $this->db()->col["name"], "country_iso_a2" => $this->db()->col["iso_a2"], "country_iso_a3" => $this->db()->col["iso_a3"], "country_iso_number" => $this->db()->col["iso_number"], "zip" => $this->_settings["ShippingOriginCountry"]=="1"?substr(trim($this->_settings["ShippingOriginZip"]), 0, 5):trim($this->_settings["ShippingOriginZip"]));
		}
		else
		{
			$address_from = array("country" => "United States", "country_iso_a2" => "US", "country_iso_a3" => "USA", "country_iso_number" => "840", "zip" => $this->_settings["ShippingOriginCountry"]=="1"?substr(trim($this->_settings["ShippingOriginZip"]), 0, 5):trim($this->_settings["ShippingOriginZip"]));
		}
		////////////////////////////////////////
		//prepare to(destination) shipping address
		$this->db()->query("
			SELECT 
				".DB_PREFIX."orders.shipping_name, 
				".DB_PREFIX."orders.shipping_address1,
				".DB_PREFIX."orders.shipping_address2,
				".DB_PREFIX."orders.shipping_city,
				".DB_PREFIX."orders.shipping_country,
				".DB_PREFIX."orders.shipping_zip,
				".DB_PREFIX."countries.name AS country_name, 
				".DB_PREFIX."countries.iso_a2, 
				".DB_PREFIX."countries.iso_a3, 
				".DB_PREFIX."countries.iso_number,
				".DB_PREFIX."states.name AS state_name,
				".DB_PREFIX."states.short_name AS state_abbr
			FROM ".DB_PREFIX."orders 
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
			LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."orders.shipping_state
			WHERE oid='".$this->oid."'
		");
		$this->db()->moveNext();
		$address_to = array(
			"name" => $this->db()->col["shipping_name"],
			"address1" => $this->db()->col["shipping_address1"],
			"address2" => $this->db()->col["shipping_address2"],
			"city" => $this->db()->col["shipping_city"],
			"state" => $this->db()->col["state_name"],
			"state_abbr" => $this->db()->col["state_abbr"],
			"country" => $this->db()->col["country_name"],
			"country_iso_a2" => $this->db()->col["iso_a2"], 
			"country_iso_a3" => $this->db()->col["iso_a3"], 
			"country_iso_number" => $this->db()->col["iso_number"], 
			"zip" => $this->db()->col["shipping_country"]=="1"?substr(trim($this->db()->col["shipping_zip"]), 0, 5):trim($this->db()->col["shipping_zip"])
		);
		
		///////////////////////////////////
		// Dollar Days
		if ($shipping_price_dollar_days > 0)
		{
			$shipping_amount_dollar_days = $shipping_price_dollar_days / 100 * $dollar_days_shipping_percent;
			if ($shipping_amount_dollar_days < $dollar_days_shipping_min_amount)
			{
				$shipping_amount_dollar_days = 	$dollar_days_shipping_min_amount;
			}
			//$product_level_method_used = true;
			$shipping_price_at_product_level = $shipping_price_at_product_level + $shipping_amount_dollar_days;
		}
		
		/////////////////////////////////
		// Doba
		if (count($doba_items) > 0)
		{ 
			$_SESSION["doba_items_in_order"] = true;
			require_once("content/classes/class.doba_order.php");
			$dobaOrder = new DobaOrderAPI($this->_settings, $this->_db);
				
		 	$shipping_amount_doba = $dobaOrder->getShippingQuote($address_to, $doba_items);
			if ($shipping_amount_doba !== false)
			{
				//$product_level_method_used = true;
				$shipping_price_at_product_level = $shipping_price_at_product_level + $shipping_amount_doba;
			}
		}
		
		//update shipping_price in orders_content
		if ($product_level_method && count($order_content_shipping_price))
		{
			foreach ($order_content_shipping_price as $ocid=>$item)
			{
				$this->db()->reset();
				$this->db()->assignStr("is_shipping_price", "Yes");
				$this->db()->assignStr("shipping_price", $item["shipping_price"]);
				$this->db()->update(DB_PREFIX."orders_content", "WHERE ocid='".intval($ocid)."'");
			}
		}
		
		
		if ($this->promoDiscountAmount > 0)
		{
			$promo_discount_percent = $this->promoDiscountType == "percent" ? $this->promoDiscountValue : $this->promoDiscountValue / ($this->subtotalAmount / 100);
		}
		else
		{
			$promo_discount_percent = 0;
		}
	
		if ($this->discountAmount > 0)
		{
			$discount_percent = $this->discountType == "percent" ? $this->discountValue : $this->discountValue / ($this->subtotalAmount / 100);
		}
		else
		{
			$discount_percent = 0;
		}
	
		if ($shipping_amount_for_price_based_exists)
		{
			$shipping_amount_for_price_based = 
				$shipping_amount_for_price_based - 
				$shipping_amount_for_price_based / 100 * $promo_discount_percent -
				$shipping_amount_for_price_based / 100 * $discount_percent;
		}
		
		$this->db()->reset();
		if ($this->shippingCarrierID == "custom" && $this->shippingMethodID == "product_level")
		{
			$result_shipping_price = $shipping_price_at_product_level;
		}
		else
		{
			////////////////////////////////////////
			//try to calc
		
			$error_message = "";
			$result_shipping_price = getShippingPrice($this->_db, $address_from, $address_to, 
				$shipping_weight, 
				$shipping_items_count,
				$shipping_amount_for_price_based, //price based
				$this->shippingSSID,
				$error_message
			) + $shipping_price_at_product_level;
		
			if ($error_message != "")
			{
				$this->shippingAmount = 0.00;
				$this->db()->assignStr("shipping_carrier_id", "custom");
				$this->db()->assignStr("shipping_method_id", "disabled");
			}
		}
		
		$this->shippingAmount = $result_shipping_price;
			
		//check is there handling fee included into shipping amount
		if (!$this->handlingSeparated)
		{
			$this->shippingAmount = $this->shippingAmount + $this->handlingAmount;
		}
		
		$this->shippingAmount = round($this->shippingAmount, 2);
		
		//tax-relates
		$this->shippingTaxable = $this->_settings["ShippingTaxable"];
		$this->shippingTaxClassId = $this->_settings["ShippingTaxClassId"];
		
		//get tax rate
		if ($this->shippingAmount > 0 && $this->shippingTaxable && array_key_exists($this->shippingTaxClassId, $this->taxRates))
		{
			$this->shippingTaxRate = $this->taxRates[$this->shippingTaxClassId]["tax_rate"];
			$this->shippingTaxDescription = $this->taxRates[$this->shippingTaxClassId]["description"];
			if ($this->handlingSeparated)
			{
				$this->shippingTaxAmount = round($this->shippingAmount / 100 * $this->shippingTaxRate, 2);
			}
			else
			{
				$this->shippingTaxAmount = round(($this->shippingAmount - $this->handlingAmount) / 100 * $this->shippingTaxRate, 2);
			}
		}
		else
		{
			$this->shippingTaxable = 0;
			$this->shippingTaxRate = 0.00;
			$this->shippingTaxDescription = "";
			$this->shippingTaxAmount = 0.00;
		}
		
		$this->db()->assignStr("shipping_taxable", $this->shippingTaxable);
		$this->db()->assignStr("shipping_tax_rate", $this->shippingTaxRate);
		$this->db()->assignStr("shipping_tax_description", $this->shippingTaxDescription);
		$this->db()->assignStr("shipping_tax_amount", $this->shippingTaxAmount);
		$this->db()->assignStr("shipping_amount", $this->shippingAmount);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid='".$this->oid."'");
			
		return $error_message != ""? false : $this->shippingAmount;
	}
	
	/**
	 * Get Shipping Address
	 * @return mixed
	 */
	public function getShippingAddress()
	{
		$this->db()->query("
			SELECT 
				".DB_PREFIX."orders.shipping_name AS name,
				".DB_PREFIX."orders.shipping_company AS company,
				".DB_PREFIX."orders.shipping_address1 AS address1,
				".DB_PREFIX."orders.shipping_address2 AS address2,
				".DB_PREFIX."orders.shipping_city AS city,
				".DB_PREFIX."orders.shipping_zip AS zip,
				".DB_PREFIX."orders.shipping_province AS province,
				".DB_PREFIX."orders.shipping_state AS state_id,
				".DB_PREFIX."countries.coid AS country_id,
				".DB_PREFIX."countries.name AS country,
				".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
				".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
				".DB_PREFIX."countries.iso_number AS country_iso_number,
				".DB_PREFIX."states.short_name AS state_abbr,
				".DB_PREFIX."states.name AS state
			FROM ".DB_PREFIX."orders 
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
			LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."orders.shipping_state
			WHERE oid='".$this->oid."'
		");
		if ($this->db()->moveNext())
		{
			$this->shippingAddress = $this->db()->col;
			if ($this->_settings["TaxAddress"] == "shipping")
			{
				$this->taxCountryId = $this->shippingAddress["country_id"];
				$this->taxStateId = $this->shippingAddress["state_id"];
			}
			return $this->db()->col;
		}
		return false;
	}

	/**
	 * Get Order Items
	 * @return mixed
	 */
	public function getOrderItems()
	{
		if ($this->oid)
		{
			$this->db()->query("
				SELECT 
					".DB_PREFIX."products.*,
					IF(".DB_PREFIX."products.url_custom='',".DB_PREFIX."products.url_default,".DB_PREFIX."products.url_custom) AS product_url,
					".DB_PREFIX."orders_content.product_sub_id,
					".DB_PREFIX."orders_content.ocid,
					".DB_PREFIX."orders_content.price AS product_price,
					".DB_PREFIX."orders_content.options,
					".DB_PREFIX."orders_content.price * ".DB_PREFIX."orders_content.quantity AS product_total,
					".DB_PREFIX."orders_content.quantity,
					".DB_PREFIX."orders_content.is_taxable,
					".DB_PREFIX."orders_content.tax_rate,
					".DB_PREFIX."orders_content.tax_description,
					".DB_PREFIX."orders_content.weight AS product_weight,
					".DB_PREFIX."orders_content.is_gift,
					".DB_PREFIX."orders_content.free_shipping AS product_free_shipping,
					".DB_PREFIX."orders_content.digital_product_key,
					".DB_PREFIX."catalog.name AS cat_name,
					IF(".DB_PREFIX."catalog.url_custom='',".DB_PREFIX."catalog.url_default,".DB_PREFIX."catalog.url_custom) AS cat_url				
				FROM ".DB_PREFIX."orders_content
				INNER JOIN ".DB_PREFIX."products ON ".DB_PREFIX."products.pid=".DB_PREFIX."orders_content.pid
				LEFT JOIN ".DB_PREFIX."catalog ON ".DB_PREFIX."products.cid=".DB_PREFIX."catalog.cid					
				WHERE
					".DB_PREFIX."orders_content.oid='".$this->oid."' 
				ORDER BY ocid
			");
			if ($this->db()->numRows()>0)
			{
				$a = $this->db()->getRecords(); 
			
				$this->subtotalAmountWithTax = 0;
				for ($i=0; $i<count($a); $i++)
				{
					if ($this->_settings["DisplayPricesWithTax"] == "YES" && $a[$i]["is_taxable"] == "Yes")
					{
						$_price = $a[$i]["product_price"];
						if ($a[$i]["tax_rate"] >= 0)
						{
							$_price = $_price + ($_price / 100 * $a[$i]["tax_rate"]);
						}
						$a[$i]["product_price"] = $_price;
						$a[$i]["product_total"] = $_price * $a[$i]["quantity"];
					}
					
					$this->subtotalAmountWithTax = $this->subtotalAmountWithTax + $a[$i]["product_total"];				
					
					$a[$i]["url"] = getProductUrl($a[$i]["product_url"], $a[$i]["pid"], $a[$i]["cid"]);
					$a[$i]["cat_url"] = getCatalogUrl($a[$i]["cat_url"], $a[$i]["cid"]);
					//inventory, min / max order
					if ($a[$i]["inventory_control"] == "Yes")
					{
						$a[$i]["allowed_max"] = $a[$i]["max_order"] > 0 ? ($a[$i]["max_order"] > $a[$i]["stock"] ? $a[$i]["stock"] : $a[$i]["max_order"]) : $a[$i]["stock"];
					}
					else
					{
						$a[$i]["allowed_max"] = 0;
					}
					
					//parse options
					$s = explode("\n", $a[$i]["options"]);
					for ($j = 0; $j<count($s); $j++)
					{
						$pos = strpos($s[$j], " : ");
						$pos2 = strpos($s[$j], "(");
                                               if (!$pos2 ) $pos2 = strlen($s[$j]);
						else $pos2 = $pos2 - $pos - 3;
						if ($pos === false)
						{
							//do nothing
						}
						else
						{
							$a[$i]["options_parsed"][] = array(
								"name" => trim(substr($s[$j], 0, $pos)),
								"value" => trim(substr($s[$j], $pos + 3, $pos2)) //"value" => trim(substr($s[$j], $pos + 3, strlen($s[$j])-$pos -3))
							);
						}
					}
				}
				$this->items = $a;
				return $a;
			}
		}
		return false;
	}
		
	/**
	 * Get order items with additional info
	 * @return mixed
	 */
	public function getOrderItemsExtended()
	{
		$result = $this->getOrderItems();
		if ($result)
		{
			if (!class_exists("ShoppingCartProducts"))
			{
				require_once("content/classes/class.products.php");
			}
			$pr = new ShoppingCartProducts($this->_db, $this->_settings);
			
			for ($i=0; $i<count($result); $i++)
			{
				//those functions already called in getOrderItems
				//$result[$i]["url"] = getProductUrl($result[$i]["title"], $result[$i]["pid"], $result[$i]["cid"]);
				//$result[$i]["cat_url"] = getCatalogUrl($result[$i]["cat_url"], $result[$i]["cid"], "1");
				$result[$i]["thumb"] = $pr->getProductThumb($result[$i]["product_id"], $result[$i]["image_location"], $result[$i]["image_url"]);
				$result[$i]["image"] = $pr->getProductImage($result[$i]["product_id"], $result[$i]["image_location"], $result[$i]["image_url"]);					
				if ($result[$i]["image"])
				{
					$result[$i]["thumb"] = $thumb_path = $pr->getProductThumb($result[$i]["product_id"], $result[$i]["image_location"], $result[$i]["image_url"]);
					$result[$i]["thumb"] = $result[$i]["thumb"]?$result[$i]["thumb"]:$result[$i]["image"];
					
					if ($result[$i]["image_location"] == "Web" || $this->_settings["CatalogThumbType"] == "Square")
					{
						$result[$i]["thumb_size"] = ' width="'.$this->_settings["CatalogThumbSize"].'" height="'.$this->_settings["CatalogThumbSize"].'" ';
					}
					else
					{
						$is = @getImageSize($thumb_path);
						if ($is)
						{
							if ($is[0] > $is[1])
							{
								$result[$i]["thumb_size"] = ' width="'.$this->_settings["CatalogThumbSize"].'" ';
							}
							else
							{
								$result[$i]["thumb_size"] = ' height="'.$this->_settings["CatalogThumbSize"].'" ';
							}
						}
					}
				}
				else
				{
					$result[$i]["thumb"] = false;
				}
			}
		}
		return $result;
	}
		
	/**
	 * Get order data
	 * @return mixed
	 */
	public function getOrderData()
	{
		if ($this->oid)
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."orders WHERE oid='".$this->oid."'");
			if ($this->db()->moveNext())
			{
				$order_data = $this->db()->col;
				
				//ORDER NUMBER
				$this->order_num = $order_data["order_num"];
				
				//SUBTOTAL
				$this->subtotalAmount = $order_data["subtotal_amount"];
				
				//DISCOUNT
				$this->discountAmount = $order_data["discount_amount"];
				$this->discountValue = $order_data["discount_value"];
				$this->discountType = $order_data["discount_type"];
				
				//PROMO DISCOUNT
				$this->promoDiscountAmount = $order_data["promo_discount_amount"];
				$this->promoDiscountValue= $order_data["promo_discount_value"];
				$this->promoDiscountType = $order_data["promo_discount_type"];
				
				//HANDLING
				$this->handlingSeparated = $order_data["handling_separated"];
				$this->handlingFee = $order_data["handling_fee"];
				$this->handlingFeeType = $order_data["handling_fee_type"];
				
				$this->handlingText = $order_data["handling_text"];
				$this->handlingAmount = $order_data["handling_amount"];
				$this->handlingTaxable = $order_data["handling_taxable"];
				$this->handlingTaxRate = $order_data["handling_tax_rate"];
				$this->handlingTaxAmount = $order_data["handling_tax_amount"];
				$this->handlingTaxDescription = $order_data["handling_tax_description"];
				
				//SHIPPING
				$this->shippingIsFree = $this->isShippingFree()?1:0;
				$this->shippingDigital = $order_data["shipping_digital"];
				$this->shippingCompleted = $order_data["shipping_completed"];
				$this->shippingSSID = $order_data["shipping_ssid"];
				$this->shippingCarrierID = $order_data["shipping_carrier_id"];
				$this->shippingMethodID = $order_data["shipping_method_id"];
				$this->shippingCMName = $order_data["shipping_cm_name"];
				$this->shippingAmount = $order_data["shipping_amount"];
				
				$this->shippingTaxable = $order_data["shipping_taxable"];
				$this->shippingTaxRate = $order_data["shipping_tax_rate"];
				$this->shippingTaxAmount = $order_data["shipping_tax_amount"];
				$this->shippingTaxDescription = $order_data["shipping_tax_description"];
				
				//BUY SAFE
				$this->buysafeWantsBond = $order_data["buysafe_wants_bond"];
				$this->buysafeBondAmount = $order_data["buysafe_bond_amount"];
				
				//TAX
				//set new address to calc tax
				if ($this->_settings["TaxAddress"] == "Shipping" && !$this->shippingDigital && $order_data["shipping_country"] > 0)
				{
					$this->taxCountryId = $order_data["shipping_country"];
					$this->taxStateId = $order_data["shipping_state"];
				}
				$this->taxRates = $this->getTaxRates();
				$this->taxAmount = $this->getTaxAmount(); //$order_data["tax_amount"];
				//$this->subtotalAmountWithTax = $this->subtotalAmount + $this->taxAmount;
				
				//PAYMENT
				$this->paymentIsRealtime = $order_data["payment_is_realtime"];
				$this->paymentMethodId = $order_data["payment_method_id"];
				$this->paymentMethodName = $order_data["payment_method_name"];
				$this->paymentGatewayId = $order_data["payment_gateway_id"];
				
				//OTHER STUFF
				$this->giftMessage = trim($order_data["gift_message"]);
				$this->totalAmount = $this->getTotalAmount();
				
				$this->itemsCount = $this->getItemsCount();
			}
		}
	}
	
	/**
	 * Recalulate order data
	 */
	public function recalcOrderData()
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."orders WHERE oid='".$this->oid."'");
		if ($this->db()->moveNext())
		{
			$order_data = $this->db()->col;
							
			//ORDER NUMBER
			$this->order_num = $order_data["order_num"];
			
			//SUBTOTAL
			$this->subtotalAmount = $this->getSubtotalAmount();
			
			//DISCOUNT
			$this->discountAmount = $this->getDiscountAmount();
			
			//PROMO DISCOUNT
			$this->promoDiscountAmount = $this->getPromoDiscountAmount();
			$this->promoDiscountAmount = $order_data["promo_discount_amount"];
			$this->promoDiscountValue= $order_data["promo_discount_value"];
			$this->promoDiscountType = $order_data["promo_discount_type"];
			
			//HANDLING
			$this->handlingAmount = $this->getHandlingAmount();
			
			//SHIPPING
			$this->shippingIsFree = $this->isShippingFree() ? 1 : 0;
			$this->shippingDigital = $order_data["shipping_digital"];
			$this->shippingCompleted = $order_data["shipping_completed"];
			$this->shippingSSID = $order_data["shipping_ssid"];
			$this->shippingCarrierID = $order_data["shipping_carrier_id"];
			$this->shippingMethodID = $order_data["shipping_method_id"];
			$this->shippingCMName = $order_data["shipping_cm_name"];
			$this->shippingAmount = $order_data["shipping_amount"];
			
			$this->shippingTaxable = $order_data["shipping_taxable"];
			
			//TAX
				//set new address to calc tax
			if ($this->_settings["TaxAddress"] == "Shipping" && !$this->shippingDigital && $order_data["shipping_country"] > 0)
			{
				$this->taxCountryId = $order_data["shipping_country"];
				$this->taxStateId = $order_data["shipping_state"];
			}
			$this->taxRates = $this->getTaxRates();
			$this->taxAmount = $this->getTaxAmount(); //$order_data["tax_amount"];
			//$this->subtotalAmountWithTax = $this->subtotalAmount + $this->taxAmount;
					
			$this->shippingTaxRate = $order_data["shipping_tax_rate"];
			$this->shippingTaxAmount = $order_data["shipping_tax_amount"];
			$this->shippingTaxDescription = $order_data["shipping_tax_description"];
					
			//PAYMENT
			$this->paymentIsRealtime = $order_data["payment_is_realtime"];
			$this->paymentMethodId = $order_data["payment_method_id"];
			$this->paymentMethodName = $order_data["payment_method_name"];
			$this->paymentGatewayId = $order_data["payment_gateway_id"];
			
			//TOTAL & ITEMS COUNT
			$this->totalAmount = $this->getTotalAmount();
			$this->itemsCount = $this->getItemsCount();
		}
	}		
}

/**
 * Additional service function
 * @param $options
 * @param $price
 * @param $weight
 * @param $result
 * @param $admin_call
 * @return mixed
 */
function parseOptions($options, $price, $weight, &$result, $admin_call = false)
{
	global $settings;
	$result = array();
	$options_array = explode("\n", $options);
	for ($i=0; $i<count($options_array); $i++)
	{
		$option = trim($options_array[$i]);
		if ($option != "")
		{
			//
			$option_parts = array();
			$option_parts = explode("(", $option);
			$option_name = trim($option_parts[0]);
			$option_price = $price;
			$option_difference = 0;
			$option_weight = $weight;
			$option_modifier = "";
			$option_value = 0;
			$option_value_type = "amount";
			$weight_option_value = 0;
			$weight_option_value_type = "amount";
			$weight_difference = 0;
			if (count($option_parts)>1)
			{
				//check price modifier
				$modifier = trim($option_parts[1]);
				if (strlen($modifier) > 2)
				{
					if ($modifier[strlen($modifier)-1] == ")")
					{
						$modifier = trim(substr($modifier, 0, strlen($modifier)-1));
						$mod_parts = explode(",", $modifier);
						
						$price_modifier = $mod_parts[0];
						
						if ($price_modifier[strlen($price_modifier)-1] == "%"){
							$procentage = substr($price_modifier, 0, strlen($price_modifier)-1)*1;
							$option_difference = round($price / 100 * $procentage, 2);
							$option_price = $price + $option_difference;
							if ($option_difference != 0.00 && $procentage != 0)
							{
								$option_modifier = ($procentage > 0 ? "+" : "").$procentage."%";
							}
							else
							{
								$option_modifier = "";
							}
							$option_value = $procentage;
							$option_value_type = "percentage";
						}
						else
						{
							$price_modifier = round($price_modifier*1, 2);
							$option_difference = $price_modifier;
							$option_price = $price + $price_modifier;
							if ($option_difference != 0 && $option_price != 0)
							{
								if ($admin_call)
								{
									$option_modifier = ($price_modifier>0?"+":"-")."".getAdminPrice($price_modifier*($price_modifier>0?1:-1));
								}
								else
								{
									$option_modifier = ($price_modifier>0?"+":"-")."".GetPrice($price_modifier*($price_modifier>0?1:-1));
								}
							}
							else
							{
								$option_modifier = "";
							}
							$option_value = $price_modifier;
							$option_value_type = "amount";
						}
						
						if (count($mod_parts) > 1)
						{
							$weight_modifier = $mod_parts[1];
							if ($weight_modifier[strlen($weight_modifier)-1] == "%")
							{
								$weight_procentage = substr($weight_modifier, 0, strlen($weight_modifier)-1)*1;
								$weight_difference = $weight / 100 * $weight_procentage;
								$option_weight = $weight + $weight / 100 * $weight_procentage;
								if ($settings["ShippingShowWeight"] == "YES" && $weight_procentage != 0)
								{
									$option_modifier = (strlen(trim($option_modifier))>0?$option_modifier.", ":$option_modifier).($weight_procentage>0?"+":"").$weight_procentage."%";
								}
								$weight_option_value = $weight_procentage;
								$weight_option_value_type = "percentage";
							}
							else
							{
								$weight_modifier *=1;
								$weight_difference = $weight_modifier;
								$option_weight = $weight + $weight_modifier;
								if ($settings["ShippingShowWeight"] == "YES" && $weight_modifier != 0)
								{
									$option_modifier = (strlen(trim($option_modifier))>0?$option_modifier.", ":$option_modifier). ($weight_modifier>0?"+":"-")."".GetWeight($weight_modifier*($weight_modifier>0?1:-1));
								}
								$weight_option_value = $weight_modifier;
								$weight_option_value_type = "amount";
							}
						}
						if ($option_modifier != "")
						{
							$option_modifier = " (".$option_modifier.")";
						}
					}
				}
				
			}
			$result[$option_name] = array(
				"full"=>$option,
				"name"=>$option_name,
				"price"=>$option_price,
				"price_difference"=>$option_difference,
				"weight"=>$option_weight,
				"modifier"=>$option_modifier,
				"option_value"=>$option_value,
				"option_value_type"=>$option_value_type,
				"weight_option_value"=>$weight_option_value,
				"weight_option_value_type"=>$weight_option_value_type,
				"weight_difference"=>$weight_difference,
				"first"=>count($result)==0
			);
		}
		
	}
	return count($result)>0;
}
