<?
/*	DataCash PHP API

	$Id: DataCash_Document.php,v 1.2.2.2 2006/06/05 16:56:30 victor Exp $
	
	This module is used to build an XML document

	Usage:
		$doc = new DataCash_Document ([Root]);

	Author:	Kenny Scott
		Dave MacRae

	Copyright: DataCash Group plc 2003
	Changelog:
		dem: 2002-05-01 Added error handling to readDocumentFromFile and
				readDocumentFromString
		dem: 2002-05-21 Modified get function to return False instead
				of NULL
		rah: 2004-06-25 Corrected and cleaned up how attributes are
				stringified, and made the root-element check in
				get() case-insensitive, as the case of the string
				from get_class() seems to vary between systems.
				Bug report in RightNow ref 040616-000011

*/

$CVSid = explode (" ", "$Id: DataCash_Document.php,v 1.2.2.2 2006/06/05 16:56:30 victor Exp $");
$included_classes{basename($CVSid{1}, ".php,v")} = $CVSid{2};

class DataCash_Document {

  function DataCash_Document ( ) {

    $this->data = array();
    if (func_num_args() == 1) {
      $this->data[] = new DataCash_Element(func_get_arg(0));
    }
    $this->attributes = array();
  }

  function setRootElement (&$element) {
    $this->data = array($element);
  }

  function &getRootElement ( ) {
    return $this->data[0];
  }

  function getDocument ( ) {

    $document = "";
    $this->indent = 0;
    $this->indent_quantity = 2;
    $root = $this->data[0];
    $document = $this->getElements($root);
    return $document;

  }

  function getElements ($main_element) {


    $elements =& $main_element->getAllElements();
    $data = $this->getIndent() . "<$main_element->name";

    $attributes = $main_element->getAllAttributes();
    if ($attributes) {
      foreach ($attributes as $attr) {
	$data .= ' ' . $attr->name . '="' . $attr->value . '"';
      }
    }
    $data .= ">";

    $this->indent += $this->indent_quantity;
    if ($elements != 0) {
      $data .= "\n";
      foreach ($elements as $element) {
	$data .= $this->getElements($element);
      }
      $this->indent -= $this->indent_quantity;
      $data .= $this->getIndent();
    } else {
      $this->indent -= $this->indent_quantity;
      $data .= $main_element->data;
    }
    $data .= "</$main_element->name>\n";
    return $data;
  }

  function get ($fullpath) {

    $root =& $this->getRootElement();

    if ( strtolower(get_class( $root )) != "datacash_element" ) {
      return;
    }

    $allElements = explode(".", $fullpath);
    array_shift($allElements);
    $aElements = array(&$root);

    foreach ($allElements as $e) {
      $el = $aElements[count($aElements)-1]->getElement($e);
      if (!$el) {
	return (FALSE);
      }
      $aElements[] = &$el;
    }

    $element = $aElements[count($aElements)-1];
    if (func_num_args() > 1 && !func_get_arg(1)) {
      return $element;
    }
    return $element->getText();

  }

  function set ($fullpath, $value) {
    if ( $fullpath != "Request.Transaction.CardTxn.Card.pan" && 
	 $fullpath != "Request.Authentication.password" && 
	 $fullpath != "Request.Transaction.CardTxn.Card.Cv2Avs.cv2" ) {
      dc_log( 3, "Setting $fullpath to [$value]" );
    }
    else {
      dc_log( 3, "Setting $fullpath" );
    }
    $root =& $this->getRootElement();
    $allElements = explode(".", $fullpath);
    $addElement = array_pop($allElements);
    array_shift($allElements);
    $aElements = array(&$root);

    foreach ($allElements as $e) {
      $thisElement =& $aElements[count($aElements)-1];
      $el =& $thisElement->getElement($e);
      if (!$el) {
	$el = new DataCash_Element($e);
	$thisElement->addElement($el);
      }
      $aElements[] = &$el;
    }

    $newElement = new DataCash_Element($addElement);
    $newElement->setText($value);

    if (func_num_args() == 3) {
      $attributes = func_get_arg(2);
      foreach ($attributes as $attributeKey => $attributeValue) {
	$newAttribute = new DataCash_Attribute($attributeKey, $attributeValue);
	$newElement->addAttribute($newAttribute);
      }
    }

    $aElements[count($aElements)-1]->addElement($newElement);
  }

  function getIndent () {
    $spaces = "";
    for ($i = 0; $i < $this->indent; $i++) {
      $spaces .= " ";
    }
    return $spaces;
  }

  function destroy ( ) {
    xml_parser_free($this->xml_parser);
  }

  function startElement($parser, $name, $attrs) {

    $this->value = "";

    $element = new DataCash_Element($name);
    foreach ($attrs as $aKey => $aValue) {
      $element->addAttribute(new DataCash_Attribute($aKey, $aValue));
    }
    if (count($this->elementTree) == 0) {
      $this->setRootElement($element);
    } else {
      $this->elementTree[count($this->elementTree)-1]->addElement($element);
    }
    $this->elementTree[] = &$element;
  }

  function endElement($parser, $name) {
    if (strlen($this->value) > 0) {
      $this->elementTree[count($this->elementTree)-1]->setText($this->value);
    }
    array_pop($this->elementTree);
    $this->value = "";
  }

  function characterData ($parser, $name) {
    if (preg_match("[\w|\d]", $name)) {
      $this->value .= $name;
    }
  }

  function readDocumentFromString($input) {

    $this->elementTree = array();
    $this->data[0] = array();
    $this->value = "";
    $this->xml_parser = xml_parser_create();
    xml_set_object($this->xml_parser, $this);
    xml_parser_set_option($this->xml_parser, XML_OPTION_CASE_FOLDING, false);
    xml_set_element_handler($this->xml_parser, "startElement", "endElement");
    xml_set_character_data_handler($this->xml_parser, "characterData");

    if (!xml_parse($this->xml_parser, $input)) {
      $this->error_msg = sprintf("XML Error: %s at line %d",
				 xml_error_string(xml_get_error_code($this->xml_parser)),
				 xml_get_current_line_number($this->xml_parser));
    }
    $this->destroy();

  }

  function readDocumentFromFile($input) {

    $this->elementTree = array();
    $this->data[0] = array();
    $this->value = "";

    $this->xml_parser = xml_parser_create();
    xml_set_object($this->xml_parser, $this);
    xml_parser_set_option($this->xml_parser, XML_OPTION_CASE_FOLDING, false);
    xml_set_element_handler($this->xml_parser, "startElement", "endElement");
    xml_set_character_data_handler($this->xml_parser, "characterData");

    if (!($fp = fopen($input, "r"))) {
      $this->file_read = false;
      $this->error_msg = "Could not open XML input file $input";
    } else {
      $this->file_read = true;
      while ($data = fread($fp, 4096)) {
	if (!xml_parse($this->xml_parser, $data, feof($fp))) {
	  fclose ($fp);
	  $this->file_read = false;
	  $this->error_msg = sprintf("XML Error: %s at line %d",
				     xml_error_string(xml_get_error_code($this->xml_parser)),
				     xml_get_current_line_number($this->xml_parser));
	  return;
	}
      }
      fclose ($fp);
    }
    $this->destroy();
  }
}

?>
