<?php

require_once 'securepayau/securexml.php';

$payment_processor_id = "securepay_securexml";
$payment_processor_name = "SecurePay (Australia) SecureXML";
$payment_processor_version = "1.0.1";
$payment_processor_class = "PAYMENT_SECUREPAY_SECUREXML";
$payment_processor_type = "cc";

class PAYMENT_SECUREPAY_SECUREXML extends PAYMENT_PROCESSOR
{
	function PAYMENT_SECUREPAY_SECUREXML()
	{
		parent::PAYMENT_PROCESSOR();
		$this->id = "securepay_securexml";
		$this->name = "SecurePay (Australia) SecureXML";
		$this->version = "1.0";
		$this->class_name = "PAYMENT_SECUREPAY_SECUREXML";
		$this->type = "cc";
		$this->description = "SecurePay (Australia) SecureXML payment gateway interface";
		$this->steps = 1;
		$this->testMode = false;
		$this->support_ccs = false;
		$this->need_cc_codes = true;
		return $this;
	}
	
	function getPaymentForm($db)
	{
		$fields = parent::getPaymentForm($db);
		
		return $fields;
	}
	
	function isTestMode()
	{
		if ($this->settings_vars[$this->id."_test_mode"]["value"] === "TRUE")
		{
			$this->testMode = true;
		}
		else
		{
			$this->testMode = false;
		}
		return $this->testMode;
	}
	
	function process($db, $user, $order, $post_form)
	{
		global $settings;
		
		$this->log("is test mode returns: " . $this->isTestMode());
		$this->log("is test mode ? A : B: " . ($this->isTestMode() ? 'A' : 'B'));
		
		//check prerequisites
		//check to make sure we have curl
		if (!function_exists("curl_version"))
		{
			$this->is_error = true;
			$this->error_message = "Libary Error - CURL support not found";
			
			$this->logStart();
			$this->log($this->error_message);
			$this->logFinish();
			
			return !$this->is_error;
		}
		
		if (isset($post_form) && is_array($post_form))
		{
			//validate required data is present in post_form
			if (!isset($post_form["cc_number"]) || is_null($post_form["cc_number"]) ||
				!isset($post_form["cc_expiration_month"]) || is_null($post_form["cc_expiration_month"]) || 
				!isset($post_form["cc_expiration_year"]) || is_null($post_form["cc_expiration_year"]) ||
				!isset($this->common_vars["order_total_amount"]["value"]) || is_null($this->common_vars["order_total_amount"]["value"]) ||
				!isset($this->common_vars["order_security_id"]["value"]) || is_null($this->common_vars["order_security_id"]["value"]))
			{
				$this->is_error = true;
				$this->error_message = "Required form field not present. Contact the Site Administrator.";
				
				$this->logStart();
				$this->log($this->error_message);
				$this->logFinish();
				
				return !$this->is_error;
			}
			
			$approved = true;
			$error = '';
			$transaction_id = '';
			$type = '';
			
			//instantiate payment gateway object
			
			$mode = ($this->isTestMode() ? SECUREPAY_TEST : SECUREPAY_LIVE);
			$fmode = ($this->isTestMode() ? SECUREPAY_FRAUD_TEST : SECUREPAY_FRAUD_LIVE);
			$merchant_id = $this->settings_vars['securepay_securexml_merchant_id']['value'];
			$merchant_password = $this->settings_vars['securepay_securexml_merchant_pw']['value'];
			$preauth = $this->settings_vars[$this->id."_transaction_mode"]["value"] === "Preauthorise";
			$fraud = $this->settings_vars[$this->id."_fraudguard"]["value"] === "TRUE";
			$prefix = $this->settings_vars[$this->id."_order_prefix"]["value"];
			
			$sxml = new securexml_transaction($mode, $merchant_id, $merchant_password);
			$fsxml = new securexml_transaction($fmode, $merchant_id, $merchant_password);
			
			if ($fraud)
			{
				$ip = $_SERVER['REMOTE_ADDR'];
				$first = $this->common_vars['billing_first_name']['value'];
				$last = $this->common_vars['billing_last_name']['value'];
				$town = $this->common_vars['billing_city']['value'];
				$post = $this->common_vars['billing_zip']['value'];
				$country_b = $this->common_vars['billing_country_iso_a2']['value'];
				$country_d = $this->common_vars['shipping_country_iso_a2']['value'];
				$email = $this->common_vars['billing_email']['value'];
				$fsxml->initFraud($ip, $first, $last, $town, $post, $country_b, $country_d, $email);
			}
			
			//setup vars for txn
			$ccNumber = $post_form["cc_number"];
			$ccExpiryMonth = $post_form["cc_expiration_month"];
			$ccExpiryYear = $post_form["cc_expiration_year"];
			$ccVericationNumber = $post_form["cc_cvv2"];
			$txnAmount = $this->common_vars["order_total_amount"]["value"];
			$txnReference = $prefix . $this->common_vars["order_id"]["value"];
			
			//request payment
			
			if ($preauth && $fraud)
			{
				$type = 'FraudGuard preauthorisation';
				$approved = $fsxml->processCreditFraudCheck($txnAmount, $txnReference, $ccNumber, $ccExpiryMonth, $ccExpiryYear, $ccVericationNumber);
			}
			
			if ($approved)
			{
				if ($preauth)
				{
					$type = $type ? $type : 'Standard preauthorisation';
					$approved = $sxml->processPreauth($txnAmount, $txnReference, $ccNumber, $ccExpiryMonth, $ccExpiryYear, $ccVericationNumber);
					
					if (!$approved)
					{
						$error = $sxml->getError();
					}
					else
					{
						$transaction_id = $sxml->getResult('transaction_id');
					}
				}
				else if ($fraud)
				{
					$type = 'FraudGuard payment';
					
					$approved = $fsxml->processCreditFraud($txnAmount, $txnReference, $ccNumber, $ccExpiryMonth, $ccExpiryYear, $ccVericationNumber);
					
					if (!$approved)
					{
						$error = $fsxml->getError();
					}
					else
					{
						$transaction_id = $fsxml->getResult('transaction_id');
					}
				}
				else
				{
					$type = 'Standard payment';
					
					$approved = $sxml->processCredit($txnAmount, $txnReference, $ccNumber, $ccExpiryMonth, $ccExpiryYear, $ccVericationNumber);
					
					if (!$approved)
					{
						$error = $sxml->getError();
					}
					else
					{
						$transaction_id = $sxml->getResult('transaction_id');
					}
				}
			}
			else
			{
				$error = $fsxml->getError();
			}
			
			//check result & create transaction if approved
			if ($approved)
			{
				$response = $type . ' successful. Transaction ID: ' . $transaction_id;
				$this->createTransaction($db, $user, $order, $response, "");
				$this->is_error = false;
				$this->error_message = "";
			}
			else
			{
				$this->is_error = true;
				$this->error_message = $error;
				
				$this->logStart();
				$this->log($this->error_message);
				$this->logFinish();
			}
		}
		else
		{
			$this->is_error = true;
			$this->error_message = "Posted form variables not present. Contact the Site Administrator";
			
			$this->logStart();
			$this->log($this->error_message);
			$this->logFinish();
		}
		
		return !$this->is_error;
	}
}
