<?php
	/**
	 * Do not cache output
	 */
	header("Cache-Control: private");
	header("Content-Type: text/html");

	define("ENV", true);
	
	require_once("content/engine/engine_config.php");
	require_once("content/engine/engine_label.php");
	require_once("content/engine/engine_functions.php");
	require_once("content/engine/engine_mysql.php");
	require_once("content/engine/engine_tree.php");
	require_once("content/engine/engine_accelerate.php");
	require_once("content/engine/engine_wishlist.php");
	require_once("content/classes/class.json.php");
	require_once("content/classes/class.xml.php");
	
	/**
	 * Start code compression
	 */
	$is_accelerated = start_accelerate(true);
	
	/**
	 * Create main database connection
	 */
	$db = new DB;
	$db->init(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
	
	// load site settings in $settings array
	require_once("content/engine/engine_settings.php");
	
	/**
	 * Check demo mode
	 */
	if(!defined("DEMO_LOCK"))define("DEMO_LOCK", false);
	if(!defined("DEMO_MODE"))define("DEMO_MODE", false);
	
	/**
	 * Resolve magic quotes
	 */
	if (get_magic_quotes_gpc())
	{
		strips($_POST);
		strips($_GET);
		strips($_REQUEST);
	}
	//set_magic_quotes_runtime(0);
	
	if (isset($_GET["url"]))
	{
		require_once("content/engine/engine_url_parser.php");
	}
	
	if ($settings["buysafe_active"] == "Yes") require_once("content/classes/class.buysafe.php");
	
	/**
	 * Start session
	 */
	$session_dir = "content".DS."cache".DS."sessiondata";
	if (is_dir($session_dir) && is_writable($session_dir))
	{
		ini_set("session.save_path", $settings["GlobalServerPath"].DS.$session_dir);
	}
	session_set_cookie_params($settings["SecurityUserTimeout"]);
	session_cache_expire(round($settings["SecurityUserTimeout"]/60)); // 24 hours
	session_name("ShoppingCartSession");
	$pcsid = isset($_REQUEST["pcsid"]) ? $_REQUEST["pcsid"] : "";
	
	if(preg_match("/[0-9a-zA-Z]{1,}/", $pcsid))
	{
		session_id($pcsid);
		session_start();
		if (!isset($_SESSION["session_stored_id"]) || $_SESSION["session_stored_id"] != $pcsid)
		{
			die();
		}
	}
	else
	{
		session_start();
		$_SESSION["session_stored_id"] = session_id();
	}

	/**
	 * Init XAJAX
	 */
	define('XAJAX_DEFAULT_CHAR_ENCODING', 'utf-8');
	require_once("content/vendors/ajax/xajax.inc.php");
	$xajax = new xajax("index.php?process_ajax_request=true");
	$xajax->setCharEncoding(XAJAX_DEFAULT_CHAR_ENCODING);
	$xajax->decodeUTF8InputOff();
	$xajax->debugOff();
	$xajax->registerFunction("processAjaxAction");

	/**
	 * Currency
	 */
	require_once("content/classes/class.currencies.php");
	$currencies = new Currencies($db);
	if (isset($_POST["set_current_currency"]))
	{
		$currencies->setCurrentCurrency(intval($_POST["set_current_currency"]));
	}
	else
	{
		$currencies->getCurrentCurrency();
	}
	
	if (!isset($_SESSION["default_currency"]))
	{
		$_SESSION["default_currency"] = $currencies->getDefaultCurrency();
	}

	// load localization procedures
	require_once("content/engine/engine_localization.php");
	require_once("content/engine/engine_url_".($settings["USE_MOD_REWRITE"]=="YES" ? "rewrite" : "default").".php");
	require_once("content/engine/engine_pages.php");
	require_once("content/engine/engine_notifications.php");
	require_once("content/engine/engine_payment.php");
	require_once("content/engine/engine_shipping.php");

	/**
	 * Current page
	 */
	$p = isset($_REQUEST["p"]) ? trim($_REQUEST["p"]) : "home";
	$p = preg_match("/^([a-z0-9_-])+$/", $p) ? $p : "home";
	
	
	// FIXME: do NOT commit the following line
	// if (array_key_exists('ddmskin', $_REQUEST)) $settings['DesignSkinName'] = $_REQUEST['ddmskin'];
	
	/**
	 * DEMO Mode
	 */
	if (DEMO_MODE)
	{
		$demoModeSkins = array(
			"BigDesign"=>"BigBlue Design",
			"BlueGreen"=>"Blue Green",
			"ModernVista"=>"Modern Vista",
			"NewOrange"=>"New Orange",
			"SimpleGreen"=>"Simple Green"
		);
		
		//Set new skin
		if (isset($_REQUEST["demoModeSetSkin"]) && array_key_exists($_REQUEST["demoModeSetSkin"], $demoModeSkins))
		{
			$settings['DesignSkinName'] = $_REQUEST["demoModeSetSkin"];
			$_SESSION['demoModeSkin'] = $_REQUEST["demoModeSetSkin"];
		}
		
		//Get current skin
		$settings['DesignSkinName'] = $sn = isset($_SESSION["demoModeSkin"]) ? $_SESSION["demoModeSkin"] : $settings['DesignSkinName'];
		
		$demoModeSkinsSettings = array(
			"BigDesign"=>array("width"=>"defaultWidth", "layout"=>false),
			"BlueGreen"=>array("width"=>"doc1024x768C", "layout"=>false),
			"ModernVista"=>array("width"=>"doc1024x768C", "layout"=>false),
			"NewOrange"=>array("width"=>"doc1024x768C", "layout"=>'a:1:{s:5:"pages";a:9:{s:7:"_global";a:1:{s:5:"zones";a:2:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}s:5:"right";a:2:{i:0;a:2:{s:5:"class";s:34:"element_panels_panel-checkout-cart";s:4:"file";s:41:"templates/panels/panel-checkout-cart.html";}i:1;a:2:{s:5:"class";s:35:"element_panels_panel-catalog-search";s:4:"file";s:42:"templates/panels/panel-catalog-search.html";}}}}s:7:"catalog";a:1:{s:5:"zones";a:1:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}}}s:4:"cart";a:1:{s:5:"zones";a:0:{}}s:6:"signup";a:1:{s:5:"zones";a:1:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}}}s:28:"billing_and_shipping_address";a:1:{s:5:"zones";a:1:{s:4:"left";a:5:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}i:4;a:2:{s:5:"class";s:34:"element_panels_panel-checkout-cart";s:4:"file";s:41:"templates/panels/panel-checkout-cart.html";}}}}s:15:"shipping_method";a:1:{s:5:"zones";a:0:{}}s:7:"invoice";a:1:{s:5:"zones";a:0:{}}s:17:"one_page_checkout";a:1:{s:5:"zones";a:0:{}}s:4:"home";a:1:{s:5:"zones";a:1:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}}}}}'),
			"SimpleGreen"=>array("width"=>"doc1024x768C", "layout"=>false)
		);
		
		if ($demoModeSkinsSettings[$sn]["width"]) $settings["LayoutSiteWidth"] = $demoModeSkinsSettings[$sn]["width"];
		if ($demoModeSkinsSettings[$sn]["layout"]) $settings["LayoutElements"] = $demoModeSkinsSettings[$sn]["layout"];
		//set skin settings
		
		view()->assign("demoModeSkins", $demoModeSkins);
		view()->assign("demoModeCurrentSkin", $settings['DesignSkinName']);
	}
	
	/**
	 * Init smarty
	 */
	view()->debugging = false;
	view()->use_sub_dirs = false;
	view()->assign("isDemoMode", DEMO_MODE);
	
	// FIXME: Assemble through backend only
	if (
		!is_dir('content/cache/skins/'.escapeFileName($settings['DesignSkinName'])) || 
		(isset($_SESSION['admin_auth_id']) && $_SESSION['admin_auth_id'] && isset($_SESSION['admin_design_mode']) && $_SESSION['admin_design_mode'])
		|| defined("DEVMODE")
	)
	{
		view()->assemble(
			array(
				'content'.DS.'engine'.DS.'design'.DS, 
				'content'.DS.'skins'.DS.escapeFileName($settings['DesignSkinName']).DS,
				'content'.DS.'skins'.DS.'_custom'.DS.'skin'.DS,
			), 
			'content'.DS.'cache'.DS.'skins'.DS.escapeFileName($settings['DesignSkinName']).DS
		);
	}

	/**
	 * Languages
	 */
	require_once("content/classes/class.languages.php");
	$languages = new Language($db);
	
	view()->assign("languages", $active_languages = $languages->getActiveLanguages());
	view()->assign("languages_count", count($active_languages));
	
	//check / set language for a first time
	if (!isset($_SESSION["current_language_id"]))
	{
		$def_lang = $languages->getDefaultLanguage();
		$_SESSION["current_language_id"] = $def_lang["language_id"];
	}
	//change current active language
	if (isset($_POST["set_language"]))
	{
		if (array_key_exists(intval($_POST["set_language"]), $active_languages))
		{
			$_SESSION["current_language_id"] = intval($_POST["set_language"]);
		}
	}	
	
	//check / set current active language
	$current_language = $languages->getActiveLanguageById($_SESSION["current_language_id"]);
	view()->assign("current_language", $current_language);
	view()->assign("msg", $msg = $languages->getDictionary($current_language["code"]));
	view()->prependJavascript($languages->getJavascriptCache($current_language["code"]));
		
	// pass settings to smarty
	foreach($settings as $key=>$value) view()->assign($key, $value);

	// by default user is unknown
	view()->assign("auth_ok", "no");
	view()->assign("current_page", $p);

	// assign main URLs
	// some links will always point to http URL (in emails for example)
	view()->assign("url_http", $url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]);
	view()->assign("url_https", $url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]);
	view()->assign("url_dinamic", $url_dinamic = ($settings["SecurityMode"] == "STANDARD"
			? $settings["GlobalHttpUrl"]
			: $settings["GlobalHttpsUrl"]
		)."/".$settings["INDEX"]
	);
	
	view()->assign("url_admin", ($settings["SecurityAdminAreaMode"] == "HTTP"
			? $settings["GlobalHttpUrl"]
			: $settings["GlobalHttpsUrl"]
		)
	);
	
	// Added url request uri
	view()->assign("url_request_uri", htmlentities($_SERVER['REQUEST_URI']));
	
	
	// load user and order
	require_once("content/engine/engine_user.php");
	require_once("content/engine/engine_order.php");

	// assign most used user and cart actions
	view()->assign("USER_EXPRESS_CHECKOUT", USER_EXPRESS_CHECKOUT);
	view()->assign("USER_START_CHECKOUT", USER_START_CHECKOUT);
	view()->assign("USER_LOGIN", USER_LOGIN);
	view()->assign("USER_LOGOUT", USER_LOGOUT);
	view()->assign("USER_PASSWORD_RESET", USER_PASSWORD_RESET);
	view()->assign("USER_UPDATE_ADDRESS", USER_UPDATE_ADDRESS);
	view()->assign("USER_DELETE_ADDRESS", USER_DELETE_ADDRESS);
	view()->assign("USER_ADD_TO_WISHLIST", USER_ADD_TO_WISHLIST);
	view()->assign("USER_UPDATE_PROFILE", USER_UPDATE_PROFILE);
	view()->assign("USER_ADD_ADDRESS", USER_ADD_ADDRESS);
	view()->assign("ORDER_ADD_ITEM", ORDER_ADD_ITEM);
	view()->assign("ORDER_REMOVE_ITEM", ORDER_REMOVE_ITEM);
	view()->assign("ORDER_CLEAR_ITEMS", ORDER_CLEAR_ITEMS);
	view()->assign("ORDER_UPDATE_ITEMS", ORDER_UPDATE_ITEMS);
	view()->assign("ORDER_ONE_PAGE_CHECKOUT", ORDER_ONE_PAGE_CHECKOUT);
	view()->assign("ORDER_BILLING_AND_SHIPPING_ADDRESS", ORDER_BILLING_AND_SHIPPING_ADDRESS);
	view()->assign("ORDER_UPDATE_BILLING_AND_SHIPPING_ADDRESS", ORDER_UPDATE_BILLING_AND_SHIPPING_ADDRESS);
	view()->assign("ORDER_UPDATE_SHIPPING_METHOD", ORDER_UPDATE_SHIPPING_METHOD);
	view()->assign("ORDER_CONTINUE_SHOPPING", ORDER_CONTINUE_SHOPPING);
	view()->assign("ORDER_CHECK_PROMO", ORDER_CHECK_PROMO);
	view()->assign("ORDER_COMPLETE_ORDER", ORDER_COMPLETE_ORDER);
	view()->assign("ORDER_WISHLIST_TO_CART", ORDER_WISHLIST_TO_CART);
	view()->assign("ORDER_COMPLETE_ORDER", ORDER_COMPLETE_ORDER);
	view()->assign("ORDER_PROCESS_PAYMENT", ORDER_PROCESS_PAYMENT);
	view()->assign("ORDER_PROCESS_PAYMENT_VALIDATION", ORDER_PROCESS_PAYMENT_VALIDATION);

	/**
	 * Gift certificate
	 */
	require_once("content/classes/class.gift_certificates.php");
	view()->assign("ORDER_ADD_GIFT_CERT", ORDER_ADD_GIFT_CERT);
	view()->assign("enable_gift_cert", $settings['enable_gift_cert']);	
	
	/**
	 * Define page zones
	 */
	$pages_public = array(
		"home", "page", "catalog", "product", "cart", "site_map", "products_reviews",
		"login", "signup", "subscribe", "unsubscribe", "password_reset", "blocked", "completed",
		"email2friend", "shipping_quote", "international", "wishlist", "manage_wishlist", "gift_certificate",
		"testimonials", "smartsuggest", "404"
	);
	$pages_public_only = array("login", "signup", "password_reset", "blocked");
	$pages_protected = array(
		"profile", "orders", "order", "address_book", "address_edit",
		"billing_and_shipping_address", "shipping_method",
		"invoice", "payment_validation", "completed", "one_page_checkout"
	);
	$pages_for_https = array(
		"profile", "orders", "order", 
		"billing_and_shipping_address", "shipping_method",
		"invoice", "payment_validation", "completed", "address_book", "address_edit", "one_page_checkout",
		"cart",	"login", "signup", "password_reset"
	);

	/**
	 * Prepare dinamic URLs engine
	 */
	$CurrentUrl = ($_SERVER["SERVER_PORT"] == 443 ? "https://" :"http://").$_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
	$uHTTP = strtolower($settings["GlobalHttpUrl"]."/");
	$uHTTPS = strtolower($settings["GlobalHttpsUrl"]."/");
	if (strtolower(substr($CurrentUrl, 0, strlen($uHTTP))) == $uHTTP)
	{
		$SecurityProtocol = "http";
	}
	elseif (strtolower(substr($CurrentUrl, 0, strlen($uHTTPS))) == $uHTTPS)
	{
		$SecurityProtocol = "https";
	}
	else{
		$SecurityProtocol = "http";
	}

	if ($settings["SecurityMode"] == "STANDARD")
	{
		if ($SecurityProtocol == "http")
		{
			$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?";
			$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
			$url_base = $settings["GlobalHttpUrl"]."/";
		}
		else
		{
			$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
			$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
			$url_base = $settings["GlobalHttpsUrl"]."/";
		}
		$url_dinamic = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?";
		$url_rel = $settings["INDEX"]."?";
	}
	else
	{
		if (in_array($p, $pages_for_https))
		{
			if ($SecurityProtocol == "http")
			{
				$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_dinamic =  $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_base = $settings["GlobalHttpsUrl"]."/";
			}
			else
			{
				$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
				$url_dinamic =  $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
				$url_base = $settings["GlobalHttpsUrl"]."/";
			}
			$url_rel = $settings["INDEX"]."?";
		}
		else
		{
			if ($SecurityProtocol == "http")
			{
				$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?";
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_dinamic = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_base = $settings["GlobalHttpUrl"]."/";
			}
			else
			{
				$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&";
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
				$url_dinamic = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
				$url_base = $settings["GlobalHttpsUrl"]."/";
			}
			$url_rel = $settings["INDEX"]."?";
		}
	}
	
	view()->assign("url_http", $url_http);
	view()->assign("url_https", $url_https);
	view()->assign("url_dinamic", $url_dinamic);
	view()->assign("url_base", $url_base);
	view()->assign("url_rel", $url_rel);
	view()->assign("url_ajax", $url_ajax = (isSslMode() ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"]) . '/'.$settings['INDEX'].'?pcsid='.session_id().'&');
	view()->assign("url_ajax_admin", $url_ajax_admin = (isSslMode() ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"]));
	
	//custom fields
	require_once("content/classes/class.custom_fields.php");
	$custom_fields = new CustomFields($db);

	// load and start shopping cart engine
	require_once("content/engine/engine_logic.php");

	// handle ajax calls
	if (isset($_REQUEST["process_ajax_request"]))
	{
		require_once("content/engine/engine_ajax_processor.php");
		$xajax->processRequests();
	}
	
	// check redirects and generate global back URL
	$backurl = isset($backurl) ? $backurl : "";
	
	//complete here in case of ajax or backurl
	if ($backurl != "" || isset($_REQUEST["process_ajax_request"]))
	{
		$_SESSION["HTTP_REFERER"] = ($_SERVER["SERVER_PORT"] == 443 ? "https://" :"http://").$_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
		
		flush_accelerate($is_accelerated);
		$_mysession = $_SESSION;
		foreach ($_mysession as $_mysessionvar=>$_mysessionkey)
		{
			$_SESSION[$_mysessionvar] = $_mysessionkey;
		}
		session_write_close();
		if ($backurl != "")
		{
			header("Location: ".$backurl."\n");
		}
		die();
	}
	
	// assign user security / order vars
	view()->assign("auth_ok", $user->auth_ok?"yes":"no");
	view()->assign("auth_level", $user->auth_ok?$user->data["level"]:-1);
	view()->assign("auth_express", $user->express?"yes":"no");
	view()->assign("order", $order);
	view()->assign("order_items", $order->getOrderItems());

	//checkout start or continue URL
	if ($user->auth_ok)
	{
		$url_checkout = $url_dinamic."oa=".($settings["CheckoutProcessMode"]=="Traditional"?ORDER_BILLING_AND_SHIPPING_ADDRESS:ORDER_ONE_PAGE_CHECKOUT);
	}
	else
	{
		if ($settings["AllowCreateAccount"] == "No")
		{
			if ($settings["CheckoutProcessMode"]=="Traditional")
			{
				$url_checkout = $url_dinamic."ua=".USER_EXPRESS_CHECKOUT."&oa=".ORDER_BILLING_AND_SHIPPING_ADDRESS;
			}
			else
			{
				$url_checkout = $url_https."ua=".USER_EXPRESS_CHECKOUT."&oa=".ORDER_ONE_PAGE_CHECKOUT;
			}
		}
		else
		{
			$url_checkout = $url_dinamic."p=cart";
		}
	}
	
	$url_checkout = $url_dinamic."p=cart";
	view()->assign("url_checkout", $url_checkout);
	
	foreach ($_GET as $_name => $_value)
	{
		if ($_name != "set_language")
		{
			$backurl .= urlencode($_name)."=".urlencode($_value)."&";
		}
	}
	view()->assign("backurl", $backurl);

	//////////////////////////////
	// HERE STARTS BLACK WOOD :)
	// I started to develop this application 5 years ago.
	// 5 years. It is pretty big kid now.
	
	//////////////////////////////////////////////////////////
	// DISPLAY PART

	//catalog menu	
	if (isset($_REQUEST["parent"]))
	{
		$parent = intval($_REQUEST["parent"]);
	}
	elseif ($settings["USE_MOD_REWRITE"] == "YES" && isset($_REQUEST["catalog_code"]))
	{
		$parts = explode("-", $_REQUEST["catalog_code"]);
		$parent = count($parts) == 3 ? intval($parts[1]) : "0";
	}
	else
	{
		$parent = "0";
	}

	require_once("content/classes/class.catalog.php");
	
	$categories = new ShoppingCartCatalog($db);
	$categories_tree = $categories->getParsedTree($parent);
	view()->assign("categories", $categories->tree);
	
	//manufacturers
	if ($settings["CatalogManufacturers"] == "YES")
	{
		require_once("content/classes/class.manufacturers.php");
		$manufacturers = new Manufacturers($db);
		$manufacturers_list = $manufacturers->getManufacturers(true);
		if ($manufacturers_list)
		{
			view()->assign("manufacturers", $manufacturers_list);
			$is_manufacturers = true;
		}
		else
		{
			view()->assign("manufacturers", false);
			$is_manufacturers = false;
		}
	}
	else
	{
		view()->assign("manufacturers", false);
		$is_manufacturers = false;
	}

	view()->assign("current_currency", $_SESSION["current_currency"]);
	$currencies_list = $currencies->getCurrencies();
	if ($currencies_list)
	{
		view()->assign("currencies", $currencies_list);
		view()->assign("currencies_count", count($currencies_list));
		$is_currencies = true;
	}
	else
	{
		view()->assign("currencies", array());
		view()->assign("currencies_count", 1);
		$is_currencies = false;
	}

	//bestsellers
	if ($settings["CatalogBestsellersAvailable"] == "YES")
	{
		require_once("content/classes/class.products.php");
		$params = $settings;
		if ($parent)
		{
			if (in_array($parent, array_keys($categories->tree)))
			{
				$params["categories"][] = $parent;
				if (is_array($categories->tree[$parent]["children"]))
				{
					reset($categories->tree[$parent]["children"]);
					foreach ($categories->tree[$parent]["children"] as $id=>$node)
					{
						$params["categories"][] = $id;
					}
				}
			}
		}
		else
		{
			$params["categories"][] = 0;
		}
		$bs = new ShoppingCartProducts($db, $settings);
		$bs->taxRates = $order->taxRates;
		view()->assign("bestsellers", $bs->getItemsBestSellers($params, $user->level));
		view()->assign("bestsellers_view", $settings["CatalogBestsellersView"]);		
		$is_bestsellers = $bs->bestsellersCount > 0;
	}
	else
	{
		view()->assign("bestsellers", false);//
		$is_bestsellers = false;
	}

	if ($user->auth_ok)
	{
		view()->assign("user_name", gs($user->data["fname"]." ".$user->data["lname"]));
	}

	//top & bottom links
	$page_id = isset($_REQUEST["page_id"]) ? $_REQUEST["page_id"] : "";
	view()->assign("pages_links", $pages_links = getPagesLinks($db, $p=="page"?$page_id:""));

	//recent items
	require_once("content/classes/class.recent.php");
	$recentItems = new ShoppingCartRecentItems($settings);
	view()->assign("recent_items", $recentItems->getItems());

	view()->assign("designImages", view()->getImages());

	view()->assign("label_app_name", $label_app_name);
	view()->assign("label_home_site", $label_home_site);

	$meta_keywords = $settings["SearchMetaKeywords"];
	$meta_title = $settings["SearchTitle"];
	$meta_description = $settings["SearchMetaDescription"];

	// check page exists and page protection
	if (in_array($p, $pages_public) || in_array($p, $pages_protected))
	{
		$body = "content/public/public_".$p.".php";
		if (in_array($p, $pages_protected) && (!$user->auth_ok))
		{
			$p = "auth_error";
			$body = "content/public/public_auth_error.php";
		}
	}
	else
	{
		$p = "404";
		$body = "content/public/public_404.php";
	}

	if ($user->auth_ok && in_array($p, $pages_public_only))
	{
		header("Location: ".$url_dinamic."p=cart");
	}

	view()->assign("site_map_url", getSiteMapUrl());

	//ajax javascript
	view()->assign("ajax_javascript", $xajax->getJavascript("content/vendors/ajax"));

	/**
	 * Log statistics
	 */
	require_once("content/engine/engine_statistics.php");
	if (($p != "catalog") && ($p != "product"))
	{
		LogStatistics($db, $user->id, "other", 0, "");
	}
	
	// load executable module (by $p variable)
	require($body);
	
	// assign $meta_xxx vars
	view()->assign("meta_title", $meta_title);
	view()->assign("meta_keywords", $meta_keywords);
	view()->assign("meta_description", $meta_description);
	
	// meta pre-assign
	view()->appendMetatag('Content-Type', 'text/html; charset='.$msg['common']['charset'], true);
	view()->appendMetatag('keywords', $meta_keywords);
	view()->appendMetatag('description', $meta_description);
	view()->setTitle($meta_title);
	
	// forces IE7 compatibility mode for IE8 browsers
	view()->appendMetatag('X-UA-Compatible', 'IE=EmulateIE7', true);
	
	// sets the metatags for use with designmode
	view()->appendMetatag('ddm-page', htmlentities($p));
	view()->appendMetatag('ddm-lang', $current_language["code"]);
	
	$doRender = ((array_key_exists('norender', $_GET) && $_GET['norender'] == 'true') === false);
	
	require_once("content/engine/engine_design_mode.php");

	if (in_array($p, array("completed", "invoice", "payment_validation", "cart", "billing", "one_page_checkout")))
	{
		view()->force_compile = true;
	}

	/**
	 * Render output
	 */
	if ($doRender)
	{
		$wrapperToRender = "default.html";
		if (isset($_SESSION['admin_auth_id']) && $_SESSION['admin_auth_id'] && isset($_SESSION['admin_design_mode']) && $_SESSION['admin_design_mode'] && isset($_REQUEST["previewMode"]))
		{
			$wrapperToRender = "preview.html";
		}
		if ($settings["StoreClosed"] == "Yes")
		{
			view()->assign("ClosedMessage", $settings["StoreClosedMessage"]);
			$wrapperToRender = "store_closed.html";
		}
		else
		{		
			$is_print_version = isset($_REQUEST["is_print_version"])?$_REQUEST["is_print_version"]:false;
			view()->assign("is_print_version", $is_print_version);
		
			if ($is_print_version == "true")
			{
				view()->assign("site_title", $settings["GlobalSiteName"]);
				$wrapperToRender = "print.html";
			}
		}
		
		view()->render("templates/wrappers/{$wrapperToRender}");
	}
	
	/**
	 * Done
	 */
	$db->done();
	
	if ($p != "email2friend")
	{
		$_SESSION["HTTP_REFERER"] = ($_SERVER["SERVER_PORT"] == "443" ? "https://" : "http://").$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
	}

	flush_accelerate($is_accelerated);
	
	session_write_close();
	
/**
 * The End
 */